@@ -3,6 +3,7 @@
  * This file is part of GtkSourceView
  *
  * Copyright (C) 2009 - Jesse van den Kieboom
+ * Copyright (C) 2013 - Sébastien Wilmet
  *
  * gtksourceview is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,32 +20,40 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+/**
+ * SECTION:completionwords
+ * @title: GtkSourceCompletionWords
+ * @short_description: A #GtkSourceCompletionProvider for the completion of words
+ *
+ * The #GtkSourceCompletionWords is an example of an implementation of
+ * the #GtkSourceCompletionProvider interface. The proposals are words
+ * appearing in the registered #GtkTextBuffer<!-- -->s.
+ */
+
 #include "gtksourcecompletionwords.h"
 #include "gtksourcecompletionwordslibrary.h"
 #include "gtksourcecompletionwordsbuffer.h"
 #include "gtksourcecompletionwordsutils.h"
+#include "gtksourceview/gtksource.h"
+#include "gtksourceview/gtksourceview-typebuiltins.h"
+#include "gtksourceview/gtksourceview-i18n.h"
 
-#include <gtksourceview/gtksourcecompletion.h>
-#include <gtksourceview/gtksourceview-i18n.h>
 #include <string.h>
 
-#define GTK_SOURCE_COMPLETION_WORDS_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_SOURCE_TYPE_COMPLETION_WORDS, GtkSourceCompletionWordsPrivate))
-
 #define BUFFER_KEY "GtkSourceCompletionWordsBufferKey"
 
-#define GET_WORDS_BUFFER(buf) (((BufferBinding *)g_object_get_data(G_OBJECT(buf), BUFFER_KEY))->buffer)
-
 enum
 {
 	PROP_0,
-	
+
 	PROP_NAME,
 	PROP_ICON,
 	PROP_PROPOSALS_BATCH_SIZE,
 	PROP_SCAN_BATCH_SIZE,
 	PROP_MINIMUM_WORD_SIZE,
 	PROP_INTERACTIVE_DELAY,
-	PROP_PRIORITY
+	PROP_PRIORITY,
+	PROP_ACTIVATION
 };
 
 struct _GtkSourceCompletionWordsPrivate
@@ -55,28 +64,24 @@ struct _GtkSourceCompletionWordsPrivate
 	gchar *word;
 	gint word_len;
 	guint idle_id;
-	
+
 	GtkSourceCompletionContext *context;
 	GSequenceIter *populate_iter;
 
 	guint cancel_id;
-	
+
 	guint proposals_batch_size;
 	guint scan_batch_size;
 	guint minimum_word_size;
-	
+
 	GtkSourceCompletionWordsLibrary *library;
 	GList *buffers;
 
 	gint interactive_delay;
 	gint priority;
+	GtkSourceCompletionActivation activation;
 };
 
-typedef struct
-{
-	GObjectClass parent_class;
-} GscProposalWordsClass;
-
 typedef struct
 {
 	GtkSourceCompletionWords *words;
@@ -85,11 +90,10 @@ typedef struct
 
 static void gtk_source_completion_words_iface_init (GtkSourceCompletionProviderIface *iface);
 
-GType gsc_proposal_words_get_type (void);
-
 G_DEFINE_TYPE_WITH_CODE (GtkSourceCompletionWords,
 			 gtk_source_completion_words,
 			 G_TYPE_OBJECT,
+			 G_ADD_PRIVATE (GtkSourceCompletionWords)
 			 G_IMPLEMENT_INTERFACE (GTK_SOURCE_TYPE_COMPLETION_PROVIDER,
 				 		gtk_source_completion_words_iface_init))
 
@@ -99,7 +103,7 @@ gtk_source_completion_words_get_name (GtkSourceCompletionProvider *self)
 	return g_strdup (GTK_SOURCE_COMPLETION_WORDS (self)->priv->name);
 }
 
-static GdkPixbuf * 
+static GdkPixbuf *
 gtk_source_completion_words_get_icon (GtkSourceCompletionProvider *self)
 {
 	return GTK_SOURCE_COMPLETION_WORDS (self)->priv->icon;
@@ -113,21 +117,20 @@ population_finished (GtkSourceCompletionWords *words)
 		g_source_remove (words->priv->idle_id);
 		words->priv->idle_id = 0;
 	}
-	
+
 	g_free (words->priv->word);
 	words->priv->word = NULL;
-	
+
 	if (words->priv->context != NULL)
 	{
 		if (words->priv->cancel_id)
 		{
-			g_signal_handler_disconnect (words->priv->context, 
+			g_signal_handler_disconnect (words->priv->context,
 			                             words->priv->cancel_id);
 			words->priv->cancel_id = 0;
 		}
-	
-		g_object_unref (words->priv->context);
-		words->priv->context = NULL;
+
+		g_clear_object (&words->priv->context);
 	}
 }
 
@@ -137,21 +140,21 @@ add_in_idle (GtkSourceCompletionWords *words)
 	guint idx = 0;
 	GList *ret = NULL;
 	gboolean finished;
-	
+
 	if (words->priv->populate_iter == NULL)
 	{
-		words->priv->populate_iter = 
+		words->priv->populate_iter =
 			gtk_source_completion_words_library_find_first (words->priv->library,
 			                                                words->priv->word,
 			                                                words->priv->word_len);
 	}
 
-	while (idx < words->priv->proposals_batch_size && 
+	while (idx < words->priv->proposals_batch_size &&
 	       words->priv->populate_iter)
 	{
-		GtkSourceCompletionWordsProposal *proposal = 
+		GtkSourceCompletionWordsProposal *proposal =
 				gtk_source_completion_words_library_get_proposal (words->priv->populate_iter);
-		
+
 		/* Only add non-exact matches */
 		if (strcmp (gtk_source_completion_words_proposal_get_word (proposal),
 		            words->priv->word) != 0)
@@ -159,16 +162,16 @@ add_in_idle (GtkSourceCompletionWords *words)
 			ret = g_list_prepend (ret, proposal);
 		}
 
-		words->priv->populate_iter = 
+		words->priv->populate_iter =
 				gtk_source_completion_words_library_find_next (words->priv->populate_iter,
 		                                                               words->priv->word,
 		                                                               words->priv->word_len);
 		++idx;
 	}
-	
+
 	ret = g_list_reverse (ret);
 	finished = words->priv->populate_iter == NULL;
-	
+
 	gtk_source_completion_context_add_proposals (words->priv->context,
 	                                             GTK_SOURCE_COMPLETION_PROVIDER (words),
 	                                             ret,
@@ -185,57 +188,24 @@ add_in_idle (GtkSourceCompletionWords *words)
 	return !finished;
 }
 
-static gboolean
-valid_word_char (gunichar ch, 
-                 gpointer data)
-{
-	return g_unichar_isprint (ch) && (ch == '_' || g_unichar_isalnum (ch));
-}
-
-static gboolean
-valid_start_char (gunichar ch,
-                  gpointer data)
-{
-	return !g_unichar_isdigit (ch);
-}
-
 static gchar *
-get_word_at_iter (GtkTextIter    *iter,
-                  CharacterCheck  valid,
-                  CharacterCheck  valid_start,
-                  gpointer        data)
+get_word_at_iter (GtkTextIter *iter)
 {
-	GtkTextIter end = *iter;
-	
-	if (!gtk_source_completion_words_utils_forward_word_end (iter, valid, data) ||
-	    !gtk_text_iter_equal (iter, &end))
-	{
-		return NULL;
-	}
-	
-	if (!gtk_source_completion_words_utils_backward_word_start (iter,
-	                                                            valid,
-	                                                            valid_start,
-	                                                            data))
-	{
-		return NULL;
-	}
-	
-	if (gtk_text_iter_equal (iter, &end))
-	{
-		return NULL;
-	}
-	else
-	{
-		return gtk_text_iter_get_text (iter, &end);
-	}
-}
+	GtkTextBuffer *buffer;
+	GtkTextIter start_line;
+	gchar *line_text;
+	gchar *word;
 
-static gboolean
-gtk_source_completion_words_match (GtkSourceCompletionProvider *provider,
-                                   GtkSourceCompletionContext  *context)
-{
-	return TRUE;
+	buffer = gtk_text_iter_get_buffer (iter);
+	start_line = *iter;
+	gtk_text_iter_set_line_offset (&start_line, 0);
+
+	line_text = gtk_text_buffer_get_text (buffer, &start_line, iter, FALSE);
+
+	word = _gtk_source_completion_words_utils_get_end_word (line_text);
+
+	g_free (line_text);
+	return word;
 }
 
 static void
@@ -243,24 +213,22 @@ gtk_source_completion_words_populate (GtkSourceCompletionProvider *provider,
                                       GtkSourceCompletionContext  *context)
 {
 	GtkSourceCompletionWords *words = GTK_SOURCE_COMPLETION_WORDS (provider);
+	GtkSourceCompletionActivation activation;
 	GtkTextIter iter;
 	gchar *word;
-	GtkTextBuffer *buffer;
-	GtkSourceCompletionWordsBuffer *buf;
 
 	gtk_source_completion_context_get_iter (context, &iter);
-	buffer = gtk_text_iter_get_buffer (&iter);
 
 	g_free (words->priv->word);
 	words->priv->word = NULL;
 
-	word = get_word_at_iter (&iter,
-	                         valid_word_char,
-	                         valid_start_char,
-	                         words);
+	word = get_word_at_iter (&iter);
+
+	activation = gtk_source_completion_context_get_activation (context);
 
 	if (word == NULL ||
-	    g_utf8_strlen (word, -1) < words->priv->minimum_word_size)
+	    (activation == GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE &&
+	     g_utf8_strlen (word, -1) < words->priv->minimum_word_size))
 	{
 		g_free (word);
 		gtk_source_completion_context_add_proposals (context,
@@ -270,52 +238,40 @@ gtk_source_completion_words_populate (GtkSourceCompletionProvider *provider,
 		return;
 	}
 
-	words->priv->cancel_id = 
+	words->priv->cancel_id =
 		g_signal_connect_swapped (context,
 			                  "cancelled",
-			                   G_CALLBACK (population_finished),
-			                   provider);
+			                  G_CALLBACK (population_finished),
+			                  provider);
 
 	words->priv->context = g_object_ref (context);
 
 	words->priv->word = word;
 	words->priv->word_len = strlen (word);
-	
-	buf = GET_WORDS_BUFFER (buffer);
-	gtk_text_buffer_move_mark (buffer,
-	                           gtk_source_completion_words_buffer_get_mark (buf),
-	                           &iter);
-	
+
 	/* Do first right now */
 	if (add_in_idle (words))
 	{
 		gtk_source_completion_words_library_lock (words->priv->library);
-		words->priv->idle_id = g_idle_add ((GSourceFunc)add_in_idle,
-		                                   words);
-	}	
-}
-
-static void
-remove_buffer (BufferBinding *binding)
-{
-	g_object_set_data (G_OBJECT (gtk_source_completion_words_buffer_get_buffer (binding->buffer)),
-	                   BUFFER_KEY,
-	                   NULL);
+		words->priv->idle_id = gdk_threads_add_idle ((GSourceFunc)add_in_idle,
+		                                             words);
+	}
 }
 
 static void
 gtk_source_completion_words_dispose (GObject *object)
 {
 	GtkSourceCompletionWords *provider = GTK_SOURCE_COMPLETION_WORDS (object);
-	GList *cp;
 
 	population_finished (provider);
 
-	cp = g_list_copy (provider->priv->buffers);
-	g_list_foreach (cp, (GFunc)remove_buffer, NULL);
+	while (provider->priv->buffers != NULL)
+	{
+		BufferBinding *binding = provider->priv->buffers->data;
+		GtkTextBuffer *buffer = gtk_source_completion_words_buffer_get_buffer (binding->buffer);
 
-	g_list_free (cp);
-	g_list_free (provider->priv->buffers);
+		gtk_source_completion_words_unregister (provider, buffer);
+	}
 
 	g_free (provider->priv->name);
 	provider->priv->name = NULL;
@@ -330,10 +286,10 @@ static void
 update_buffers_batch_size (GtkSourceCompletionWords *words)
 {
 	GList *item;
-	
-	for (item = words->priv->buffers; item; item = g_list_next (item))
+
+	for (item = words->priv->buffers; item != NULL; item = g_list_next (item))
 	{
-		BufferBinding *binding = (BufferBinding *)item->data;
+		BufferBinding *binding = item->data;
 		gtk_source_completion_words_buffer_set_scan_batch_size (binding->buffer,
 		                                                        words->priv->scan_batch_size);
 	}
@@ -343,8 +299,8 @@ static void
 update_buffers_minimum_word_size (GtkSourceCompletionWords *words)
 {
 	GList *item;
-	
-	for (item = words->priv->buffers; item; item = g_list_next (item))
+
+	for (item = words->priv->buffers; item != NULL; item = g_list_next (item))
 	{
 		BufferBinding *binding = (BufferBinding *)item->data;
 		gtk_source_completion_words_buffer_set_minimum_word_size (binding->buffer,
@@ -359,46 +315,53 @@ gtk_source_completion_words_set_property (GObject      *object,
                                           GParamSpec   *pspec)
 {
 	GtkSourceCompletionWords *self = GTK_SOURCE_COMPLETION_WORDS (object);
-	
+
 	switch (prop_id)
 	{
 		case PROP_NAME:
 			g_free (self->priv->name);
 			self->priv->name = g_value_dup_string (value);
-			
+
 			if (self->priv->name == NULL)
 			{
 				self->priv->name = g_strdup (_("Document Words"));
 			}
-		break;
+			break;
+
 		case PROP_ICON:
-			if (self->priv->icon)
-			{
-				g_object_unref (self->priv->icon);
-			}
-			
+			g_clear_object (&self->priv->icon);
 			self->priv->icon = g_value_dup_object (value);
-		break;
+			break;
+
 		case PROP_PROPOSALS_BATCH_SIZE:
 			self->priv->proposals_batch_size = g_value_get_uint (value);
-		break;
+			break;
+
 		case PROP_SCAN_BATCH_SIZE:
 			self->priv->scan_batch_size = g_value_get_uint (value);
 			update_buffers_batch_size (self);
-		break;
+			break;
+
 		case PROP_MINIMUM_WORD_SIZE:
 			self->priv->minimum_word_size = g_value_get_uint (value);
 			update_buffers_minimum_word_size (self);
-		break;
+			break;
+
 		case PROP_INTERACTIVE_DELAY:
 			self->priv->interactive_delay = g_value_get_int (value);
-		break;
+			break;
+
 		case PROP_PRIORITY:
 			self->priv->priority = g_value_get_int (value);
-		break;
+			break;
+
+		case PROP_ACTIVATION:
+			self->priv->activation = g_value_get_flags (value);
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
+			break;
 	}
 }
 
@@ -409,37 +372,48 @@ gtk_source_completion_words_get_property (GObject    *object,
                                           GParamSpec *pspec)
 {
 	GtkSourceCompletionWords *self = GTK_SOURCE_COMPLETION_WORDS (object);
-	
+
 	switch (prop_id)
 	{
 		case PROP_NAME:
 			g_value_set_string (value, self->priv->name);
-		break;
+			break;
+
 		case PROP_ICON:
 			g_value_set_object (value, self->priv->icon);
-		break;
+			break;
+
 		case PROP_PROPOSALS_BATCH_SIZE:
 			g_value_set_uint (value, self->priv->proposals_batch_size);
-		break;
+			break;
+
 		case PROP_SCAN_BATCH_SIZE:
 			g_value_set_uint (value, self->priv->scan_batch_size);
-		break;
+			break;
+
 		case PROP_MINIMUM_WORD_SIZE:
 			g_value_set_uint (value, self->priv->minimum_word_size);
-		break;
+			break;
+
 		case PROP_INTERACTIVE_DELAY:
 			g_value_set_int (value, self->priv->interactive_delay);
-		break;
+			break;
+
 		case PROP_PRIORITY:
 			g_value_set_int (value, self->priv->priority);
-		break;
+			break;
+
+		case PROP_ACTIVATION:
+			g_value_set_flags (value, self->priv->activation);
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
+			break;
 	}
 }
 
-static void 
+static void
 gtk_source_completion_words_class_init (GtkSourceCompletionWordsClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
@@ -448,15 +422,15 @@ gtk_source_completion_words_class_init (GtkSourceCompletionWordsClass *klass)
 
 	object_class->set_property = gtk_source_completion_words_set_property;
 	object_class->get_property = gtk_source_completion_words_get_property;
-	
+
 	g_object_class_install_property (object_class,
 	                                 PROP_NAME,
 	                                 g_param_spec_string ("name",
-	                                                      _("Name"),
+	                                                      C_("Object", "Name"),
 	                                                      _("The provider name"),
 	                                                      NULL,
 	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-	
+
 	g_object_class_install_property (object_class,
 	                                 PROP_ICON,
 	                                 g_param_spec_object ("icon",
@@ -484,7 +458,7 @@ gtk_source_completion_words_class_init (GtkSourceCompletionWordsClass *klass)
 	                                                    G_MAXUINT,
 	                                                    50,
 	                                                    G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-	
+
 	g_object_class_install_property (object_class,
 	                                 PROP_MINIMUM_WORD_SIZE,
 	                                 g_param_spec_uint ("minimum-word-size",
@@ -515,7 +489,22 @@ gtk_source_completion_words_class_init (GtkSourceCompletionWordsClass *klass)
 	                                                   0,
 	                                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
-	g_type_class_add_private (object_class, sizeof(GtkSourceCompletionWordsPrivate));
+	/**
+	 * GtkSourceCompletionWords:activation:
+	 *
+	 * The type of activation.
+	 *
+	 * Since: 3.10
+	 */
+	g_object_class_install_property (object_class,
+	                                 PROP_ACTIVATION,
+					 g_param_spec_flags ("activation",
+							     _("Activation"),
+							     _("The type of activation"),
+							     GTK_SOURCE_TYPE_COMPLETION_ACTIVATION,
+							     GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE |
+							     GTK_SOURCE_COMPLETION_ACTIVATION_USER_REQUESTED,
+							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 }
 
 static gboolean
@@ -524,18 +513,18 @@ gtk_source_completion_words_get_start_iter (GtkSourceCompletionProvider *provide
                                             GtkSourceCompletionProposal *proposal,
                                             GtkTextIter                 *iter)
 {
-	GtkTextBuffer *buffer;
-	GtkSourceCompletionWordsBuffer *buf;
-	GtkTextIter it;
-	
-	gtk_source_completion_context_get_iter (context, &it);
-	
-	buffer = gtk_text_iter_get_buffer (&it);
-	buf = GET_WORDS_BUFFER (buffer);
-	
-	gtk_text_buffer_get_iter_at_mark (buffer,
-	                                  iter,
-	                                  gtk_source_completion_words_buffer_get_mark (buf));
+	gchar *word;
+	glong nb_chars;
+
+	gtk_source_completion_context_get_iter (context, iter);
+
+	word = get_word_at_iter (iter);
+	g_return_val_if_fail (word != NULL, FALSE);
+
+	nb_chars = g_utf8_strlen (word, -1);
+	gtk_text_iter_backward_chars (iter, nb_chars);
+
+	g_free (word);
 	return TRUE;
 }
 
@@ -551,25 +540,29 @@ gtk_source_completion_words_get_priority (GtkSourceCompletionProvider *provider)
 	return GTK_SOURCE_COMPLETION_WORDS (provider)->priv->priority;
 }
 
+static GtkSourceCompletionActivation
+gtk_source_completion_words_get_activation (GtkSourceCompletionProvider *provider)
+{
+	return GTK_SOURCE_COMPLETION_WORDS (provider)->priv->activation;
+}
+
 static void
 gtk_source_completion_words_iface_init (GtkSourceCompletionProviderIface *iface)
 {
 	iface->get_name = gtk_source_completion_words_get_name;
 	iface->get_icon = gtk_source_completion_words_get_icon;
-
 	iface->populate = gtk_source_completion_words_populate;
-	iface->match = gtk_source_completion_words_match;
-
 	iface->get_start_iter = gtk_source_completion_words_get_start_iter;
 	iface->get_interactive_delay = gtk_source_completion_words_get_interactive_delay;
 	iface->get_priority = gtk_source_completion_words_get_priority;
+	iface->get_activation = gtk_source_completion_words_get_activation;
 }
 
-static void 
+static void
 gtk_source_completion_words_init (GtkSourceCompletionWords *self)
-{	
-	self->priv = GTK_SOURCE_COMPLETION_WORDS_GET_PRIVATE (self);
-	
+{
+	self->priv = gtk_source_completion_words_get_instance_private (self);
+
 	self->priv->library = gtk_source_completion_words_library_new ();
 }
 
@@ -584,16 +577,16 @@ GtkSourceCompletionWords *
 gtk_source_completion_words_new (const gchar *name,
                                  GdkPixbuf   *icon)
 {
-	return g_object_new (GTK_SOURCE_TYPE_COMPLETION_WORDS, 
-	                     "name", name, 
-	                     "icon", icon, 
+	return g_object_new (GTK_SOURCE_TYPE_COMPLETION_WORDS,
+	                     "name", name,
+	                     "icon", icon,
 	                     NULL);
 }
 
 static void
 buffer_destroyed (BufferBinding *binding)
 {
-	binding->words->priv->buffers = g_list_remove (binding->words->priv->buffers, 
+	binding->words->priv->buffers = g_list_remove (binding->words->priv->buffers,
 	                                               binding);
 	g_object_unref (binding->buffer);
 	g_slice_free (BufferBinding, binding);
@@ -615,33 +608,33 @@ gtk_source_completion_words_register (GtkSourceCompletionWords *words,
 
 	g_return_if_fail (GTK_SOURCE_IS_COMPLETION_WORDS (words));
 	g_return_if_fail (GTK_IS_TEXT_BUFFER (buffer));
-	
+
 	binding = g_object_get_data (G_OBJECT (buffer), BUFFER_KEY);
-	
+
 	if (binding != NULL)
 	{
 		return;
 	}
-	
+
 	buf = gtk_source_completion_words_buffer_new (words->priv->library,
 	                                              buffer);
-	
+
 	gtk_source_completion_words_buffer_set_scan_batch_size (buf,
 	                                                        words->priv->scan_batch_size);
 
 	gtk_source_completion_words_buffer_set_minimum_word_size (buf,
 	                                                          words->priv->minimum_word_size);
-	
+
 	binding = g_slice_new (BufferBinding);
 	binding->words = words;
 	binding->buffer = buf;
-	
-	g_object_set_data_full (G_OBJECT (buffer), 
-	                        BUFFER_KEY, 
+
+	g_object_set_data_full (G_OBJECT (buffer),
+	                        BUFFER_KEY,
 	                        binding,
 	                        (GDestroyNotify)buffer_destroyed);
 
-	words->priv->buffers = g_list_prepend (words->priv->buffers, 
+	words->priv->buffers = g_list_prepend (words->priv->buffers,
 	                                       binding);
 }
 
@@ -650,7 +643,7 @@ gtk_source_completion_words_register (GtkSourceCompletionWords *words,
  * @words: a #GtkSourceCompletionWords
  * @buffer: a #GtkTextBuffer
  *
- * Unregisters @buffer from teh @words provider.
+ * Unregisters @buffer from the @words provider.
  */
 void
 gtk_source_completion_words_unregister (GtkSourceCompletionWords *words,
@@ -658,7 +651,6 @@ gtk_source_completion_words_unregister (GtkSourceCompletionWords *words,
 {
 	g_return_if_fail (GTK_SOURCE_IS_COMPLETION_WORDS (words));
 	g_return_if_fail (GTK_IS_TEXT_BUFFER (buffer));
-	
+
 	g_object_set_data (G_OBJECT (buffer), BUFFER_KEY, NULL);
 }
-
