@@ -30,10 +30,11 @@
 #include <string.h>
 #include <gtk/gtk.h>
 
+#include "gtksourcebuffer.h"
+#include "gtksourcebuffer-private.h"
 #include "gtksourceview-i18n.h"
 #include "gtksourcelanguage.h"
 #include "gtksourcelanguage-private.h"
-#include "gtksourcebuffer.h"
 #include "gtksourceundomanager.h"
 #include "gtksourceview-marshal.h"
 #include "gtksourcestylescheme.h"
@@ -42,24 +43,76 @@
 #include "gtksourceundomanagerdefault.h"
 #include "gtksourceview-typebuiltins.h"
 #include "gtksourcemark.h"
+#include "gtksourcesearchcontext.h"
 
 /**
  * SECTION:buffer
  * @Short_description: Buffer object for #GtkSourceView
  * @Title: GtkSourceBuffer
- * @See_also: #GtkTextBuffer,#GtkSourceView
+ * @See_also: #GtkTextBuffer, #GtkSourceView
  *
- * The #GtkSourceBuffer object is the model for #GtkSourceView widgets.
- * It extends the #GtkTextBuffer object by adding features useful to display
- * and edit source code as syntax highlighting and bracket matching. It
- * also implements support for undo/redo operations.
- *
- * To create a #GtkSourceBuffer use gtk_source_buffer_new() or
- * gtk_source_buffer_new_with_language(). The second form is just a convenience
- * function which allows you to initially set a #GtkSourceLanguage.
- *
- * By default highlighting is enabled, but you can disable it with
- * gtk_source_buffer_set_highlight_syntax().
+ * <para>
+ *   The #GtkSourceBuffer object is the model for #GtkSourceView widgets.
+ *   It extends the #GtkTextBuffer object by adding features useful to display
+ *   and edit source code such as syntax highlighting and bracket matching. It
+ *   also implements support for undo/redo operations, and for the search and
+ *   replace.
+ * </para>
+ * <para>
+ *   To create a #GtkSourceBuffer use gtk_source_buffer_new() or
+ *   gtk_source_buffer_new_with_language(). The second form is just a convenience
+ *   function which allows you to initially set a #GtkSourceLanguage.
+ * </para>
+ * <para>
+ *   By default highlighting is enabled, but you can disable it with
+ *   gtk_source_buffer_set_highlight_syntax().
+ * </para>
+ * <refsect2>
+ *   <title>Undo and Redo</title>
+ *   <para>
+ *     A custom #GtkSourceUndoManager can be implemented and set with
+ *     gtk_source_buffer_set_undo_manager(). However the default implementation
+ *     should be suitable for most uses. By default, actions that can be undone or
+ *     redone are defined as groups of operations between a call to
+ *     gtk_text_buffer_begin_user_action() and gtk_text_buffer_end_user_action(). In
+ *     general, this happens whenever the user presses any key which modifies the
+ *     buffer. But the default undo manager will try to merge similar consecutive
+ *     actions, such as multiple character insertions on the same line, into one
+ *     action. But, inserting a newline starts a new action.
+ *   </para>
+ *   <para>
+ *     The default undo manager remembers the "modified" state of the buffer, and
+ *     restore it when an action is undone or redone. It can be useful in a text
+ *     editor to know whether the file is saved. See gtk_text_buffer_get_modified()
+ *     and gtk_text_buffer_set_modified().
+ *   </para>
+ * </refsect2>
+ * <refsect2>
+ *   <title>Context Classes</title>
+ *   <para>
+ *     It is possible to retrieve some information from the syntax highlighting
+ *     engine. There are currently three context classes, that are applied to
+ *     regions of a #GtkSourceBuffer:
+ *   </para>
+ *   <itemizedlist>
+ *     <listitem>
+ *       <para>
+ *         <emphasis>comment</emphasis>: the region delimits a comment;
+ *       </para>
+ *     </listitem>
+ *     <listitem>
+ *       <para>
+ *         <emphasis>string</emphasis>: the region delimits a string;
+ *       </para>
+ *     </listitem>
+ *     <listitem>
+ *       <para>
+ *         <emphasis>no-spell-check</emphasis>: the region should not be spell
+ *         checked.
+ *       </para>
+ *     </listitem>
+ *   </itemizedlist>
+ * </refsect2>
  */
 
 /*
@@ -81,6 +134,7 @@
 #define PROFILE(x)
 #endif
 
+/* For bracket matching */
 #define MAX_CHARS_BEFORE_FINDING_A_MATCH    10000
 
 #define TAG_CONTEXT_CLASS_NAME "GtkSourceViewTagContextClassName"
@@ -125,13 +179,15 @@ struct _GtkSourceBufferPrivate
 	GtkSourceUndoManager  *undo_manager;
 	gint                   max_undo_levels;
 
+	GList                 *search_contexts;
+
 	guint                  highlight_syntax : 1;
 	guint                  highlight_brackets : 1;
 	guint                  constructed : 1;
 	guint                  allow_bracket_match : 1;
 };
 
-G_DEFINE_TYPE (GtkSourceBuffer, gtk_source_buffer, GTK_TYPE_TEXT_BUFFER)
+G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceBuffer, gtk_source_buffer, GTK_TYPE_TEXT_BUFFER)
 
 static guint 	 buffer_signals[LAST_SIGNAL];
 
@@ -336,6 +392,7 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 	/**
 	 * GtkSourceBuffer::source-mark-updated:
 	 * @buffer: the buffer that received the signal
+	 * @mark: the #GtkSourceMark
 	 *
 	 * The ::source_mark_updated signal is emitted each time
 	 * a mark is added to, moved or removed from the @buffer.
@@ -392,8 +449,6 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 			  G_TYPE_NONE, 2,
 			  GTK_TYPE_TEXT_ITER,
 			  GTK_SOURCE_TYPE_BRACKET_MATCH_TYPE);
-
-	g_type_class_add_private (object_class, sizeof (GtkSourceBufferPrivate));
 }
 
 static void
@@ -439,13 +494,18 @@ set_undo_manager (GtkSourceBuffer      *buffer,
 	}
 }
 
+static void
+search_context_weak_notify_cb (GtkSourceBuffer *buffer,
+			       GObject         *search_context)
+{
+	buffer->priv->search_contexts = g_list_remove (buffer->priv->search_contexts,
+						       search_context);
+}
+
 static void
 gtk_source_buffer_init (GtkSourceBuffer *buffer)
 {
-	GtkSourceBufferPrivate *priv;
-
-	priv = G_TYPE_INSTANCE_GET_PRIVATE (buffer, GTK_SOURCE_TYPE_BUFFER,
-					    GtkSourceBufferPrivate);
+	GtkSourceBufferPrivate *priv = gtk_source_buffer_get_instance_private (buffer);
 
 	buffer->priv = priv;
 
@@ -459,7 +519,9 @@ gtk_source_buffer_init (GtkSourceBuffer *buffer)
 	priv->style_scheme = _gtk_source_style_scheme_get_default ();
 
 	if (priv->style_scheme != NULL)
+	{
 		g_object_ref (priv->style_scheme);
+	}
 }
 
 static void
@@ -483,6 +545,7 @@ static void
 gtk_source_buffer_dispose (GObject *object)
 {
 	GtkSourceBuffer *buffer;
+	GList *l;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GTK_SOURCE_IS_BUFFER (object));
@@ -503,6 +566,18 @@ gtk_source_buffer_dispose (GObject *object)
 	g_clear_object (&buffer->priv->language);
 	g_clear_object (&buffer->priv->style_scheme);
 
+	for (l = buffer->priv->search_contexts; l != NULL; l = l->next)
+	{
+		GtkSourceSearchContext *search_context = l->data;
+
+		g_object_weak_unref (G_OBJECT (search_context),
+				     (GWeakNotify)search_context_weak_notify_cb,
+				     buffer);
+	}
+
+	g_list_free (buffer->priv->search_contexts);
+	buffer->priv->search_contexts = NULL;
+
 	G_OBJECT_CLASS (gtk_source_buffer_parent_class)->dispose (object);
 }
 
@@ -522,12 +597,12 @@ gtk_source_buffer_set_property (GObject      *object,
 	{
 		case PROP_HIGHLIGHT_SYNTAX:
 			gtk_source_buffer_set_highlight_syntax (source_buffer,
-							      g_value_get_boolean (value));
+								g_value_get_boolean (value));
 			break;
 
 		case PROP_HIGHLIGHT_MATCHING_BRACKETS:
 			gtk_source_buffer_set_highlight_matching_brackets (source_buffer,
-								g_value_get_boolean (value));
+									   g_value_get_boolean (value));
 			break;
 
 		case PROP_MAX_UNDO_LEVELS:
@@ -1209,11 +1284,6 @@ gtk_source_buffer_can_redo (GtkSourceBuffer *buffer)
  * Undoes the last user action which modified the buffer.  Use
  * gtk_source_buffer_can_undo() to check whether a call to this
  * function will have any effect.
- *
- * Actions are defined as groups of operations between a call to
- * gtk_text_buffer_begin_user_action() and
- * gtk_text_buffer_end_user_action(), or sequences of similar edits
- * (inserts or deletes) on the same line.
  **/
 void
 gtk_source_buffer_undo (GtkSourceBuffer *buffer)
@@ -1266,13 +1336,6 @@ gtk_source_buffer_get_max_undo_levels (GtkSourceBuffer *buffer)
  * function, older actions will be discarded.
  *
  * If @max_undo_levels is -1, no limit is set.
- *
- * A new action is started whenever the function
- * gtk_text_buffer_begin_user_action() is called.  In general, this
- * happens whenever the user presses any key which modifies the
- * buffer, but the undo manager will try to merge similar consecutive
- * actions, such as multiple character insertions into one action.
- * But, inserting a newline does start a new action.
  **/
 void
 gtk_source_buffer_set_max_undo_levels (GtkSourceBuffer *buffer,
@@ -1525,13 +1588,27 @@ _gtk_source_buffer_update_highlight (GtkSourceBuffer   *buffer,
 				     const GtkTextIter *end,
 				     gboolean           synchronous)
 {
+	GList *l;
+
 	g_return_if_fail (GTK_SOURCE_IS_BUFFER (buffer));
 
 	if (buffer->priv->highlight_engine != NULL)
+	{
 		_gtk_source_engine_update_highlight (buffer->priv->highlight_engine,
 						     start,
 						     end,
 						     synchronous);
+	}
+
+	for (l = buffer->priv->search_contexts; l != NULL; l = l->next)
+	{
+		GtkSourceSearchContext *search_context = l->data;
+
+		_gtk_source_search_context_update_highlight (search_context,
+							     start,
+							     end,
+							     synchronous);
+	}
 }
 
 /**
@@ -2257,8 +2334,11 @@ gtk_source_buffer_remove_source_marks (GtkSourceBuffer   *buffer,
  * @iter: a #GtkTextIter.
  * @context_class: class to search for.
  *
- * Check if the class @context_klass is set on @iter.
+ * Check if the class @context_class is set on @iter.
  *
+ * See the #GtkSourceBuffer description for the list of context classes.
+ *
+ * Returns: whether @iter has the context class.
  * Since: 2.10
  **/
 gboolean
@@ -2297,6 +2377,8 @@ gtk_source_buffer_iter_has_context_class (GtkSourceBuffer   *buffer,
  *
  * Get all defined context classes at @iter.
  *
+ * See the #GtkSourceBuffer description for the list of context classes.
+ *
  * Returns: (array zero-terminated=1) (transfer full): a new %NULL
  * terminated array of context class names.
  * Use g_strfreev() to free the array if it is no longer needed.
@@ -2328,6 +2410,8 @@ gtk_source_buffer_get_context_classes_at_iter (GtkSourceBuffer   *buffer,
 		}
 	}
 
+	g_slist_free (tags);
+
 	g_ptr_array_add (ret, NULL);
 	return (gchar **) g_ptr_array_free (ret, FALSE);
 }
@@ -2344,6 +2428,8 @@ gtk_source_buffer_get_context_classes_at_iter (GtkSourceBuffer   *buffer,
  * @iter to the location of the toggle, or to the end of the buffer if no
  * toggle is found.
  *
+ * See the #GtkSourceBuffer description for the list of context classes.
+ *
  * Returns: whether we found a context class toggle after @iter
  *
  * Since: 2.10
@@ -2389,6 +2475,8 @@ gtk_source_buffer_iter_forward_to_context_class_toggle (GtkSourceBuffer *buffer,
  * @iter to the location of the toggle, or to the end of the buffer if no
  * toggle is found.
  *
+ * See the #GtkSourceBuffer description for the list of context classes.
+ *
  * Returns: whether we found a context class toggle before @iter
  *
  * Since: 2.10
@@ -2472,3 +2560,24 @@ gtk_source_buffer_get_undo_manager (GtkSourceBuffer *buffer)
 
 	return buffer->priv->undo_manager;
 }
+
+void
+_gtk_source_buffer_add_search_context (GtkSourceBuffer        *buffer,
+				       GtkSourceSearchContext *search_context)
+{
+	g_return_if_fail (GTK_SOURCE_IS_BUFFER (buffer));
+	g_return_if_fail (GTK_SOURCE_IS_SEARCH_CONTEXT (search_context));
+	g_return_if_fail (gtk_source_search_context_get_buffer (search_context) == buffer);
+
+	if (g_list_find (buffer->priv->search_contexts, search_context) != NULL)
+	{
+		return;
+	}
+
+	buffer->priv->search_contexts = g_list_prepend (buffer->priv->search_contexts,
+							search_context);
+
+	g_object_weak_ref (G_OBJECT (search_context),
+			   (GWeakNotify)search_context_weak_notify_cb,
+			   buffer);
+}
