@@ -23,9 +23,10 @@
 #define __GTK_SOURCE_COMPLETION_PIVATE_H__
 
 #include <gtk/gtk.h>
-#include <gtksourceview/gtksourcecompletion.h>
+#include <gtksourceview/gtksourcetypes.h>
 
-GtkSourceCompletion *gtk_source_completion_new (struct _GtkSourceView *source_view);
+G_GNUC_INTERNAL
+GtkSourceCompletion *gtk_source_completion_new (GtkSourceView *source_view);
 
 #endif /* __GTK_SOURCE_COMPLETION_PIVATE_H__ */
 
