@@ -27,10 +27,11 @@
  * @short_description: Completion provider interface
  *
  * You must implement this interface to provide proposals to #GtkSourceCompletion
- *
  */
 
-#include <gtksourceview/gtksourcecompletionprovider.h>
+#include "gtksourcecompletionprovider.h"
+#include "gtksourcecompletionproposal.h"
+#include "gtksourcecompletioninfo.h"
 
 typedef GtkSourceCompletionProviderIface GtkSourceCompletionProviderInterface;
 
@@ -175,7 +176,8 @@ gtk_source_completion_provider_get_icon (GtkSourceCompletionProvider *provider)
  * @provider: a #GtkSourceCompletionProvider.
  * @context: a #GtkSourceCompletionContext.
  *
- * Populate @context with proposals from @provider.
+ * Populate @context with proposals from @provider added with the
+ * gtk_source_completion_context_add_proposals() function.
  */
 void
 gtk_source_completion_provider_populate (GtkSourceCompletionProvider *provider,
@@ -231,10 +233,17 @@ gtk_source_completion_provider_match (GtkSourceCompletionProvider *provider,
  * This allows for customized widgets on a proposal basis, although in general
  * providers will have the same custom widget for all their proposals and
  * @proposal can be ignored. The implementation of this function is optional.
- * If implemented, #gtk_source_completion_provider_update_info MUST also be
- * implemented. If not implemented, the default
- * #gtk_source_completion_proposal_get_info will be used to display extra
- * information about a #GtkSourceCompletionProposal.
+ *
+ * If this function is not implemented, the default widget is a #GtkLabel. The
+ * return value of gtk_source_completion_proposal_get_info() is used as the
+ * content of the #GtkLabel.
+ *
+ * <note>
+ *   <para>
+ *     If implemented, gtk_source_completion_provider_update_info()
+ *     <emphasis>must</emphasis> also be implemented.
+ *   </para>
+ * </note>
  *
  * Returns: (transfer none): a custom #GtkWidget to show extra
  * information about @proposal.
@@ -256,8 +265,13 @@ gtk_source_completion_provider_get_info_widget (GtkSourceCompletionProvider *pro
  * @info: a #GtkSourceCompletionInfo.
  *
  * Update extra information shown in @info for @proposal.
- * This function MUST be implemented when
- * #gtk_source_completion_provider_get_info_widget is implemented.
+ *
+ * <note>
+ *   <para>
+ *     This function <emphasis>must</emphasis> be implemented when
+ *     gtk_source_completion_provider_get_info_widget() is implemented.
+ *   </para>
+ * </note>
  */
 void
 gtk_source_completion_provider_update_info (GtkSourceCompletionProvider *provider,
@@ -279,9 +293,13 @@ gtk_source_completion_provider_update_info (GtkSourceCompletionProvider *provide
  * @iter: a #GtkTextIter.
  *
  * Get the #GtkTextIter at which the completion for @proposal starts. When
- * implemented, the completion can use this information to position the
- * completion window accordingly when a proposal is selected in the completion
- * window.
+ * implemented, this information is used to position the completion window
+ * accordingly when a proposal is selected in the completion window. The
+ * @proposal text inside the completion window is aligned on @iter.
+ *
+ * If this function is not implemented, the word boundary is taken to position
+ * the completion window. See gtk_source_completion_provider_activate_proposal()
+ * for an explanation on the word boundaries.
  *
  * When the @proposal is activated, the default handler uses @iter as the start
  * of the word to replace. See
@@ -318,7 +336,7 @@ gtk_source_completion_provider_get_start_iter (GtkSourceCompletionProvider *prov
  *
  * Here is how the default activation selects the boundaries of the word to
  * replace. The end of the word is @iter. For the start of the word, it depends
- * on wheter a start iter is defined for @proposal (see
+ * on whether a start iter is defined for @proposal (see
  * gtk_source_completion_provider_get_start_iter()). If a start iter is defined,
  * the start of the word is the start iter. Else, the word (as long as possible)
  * will contain only alphanumerical and the "_" characters.
@@ -345,7 +363,7 @@ gtk_source_completion_provider_activate_proposal (GtkSourceCompletionProvider *p
  *
  * Get the delay in milliseconds before starting interactive completion for
  * this provider. A value of -1 indicates to use the default value as set
- * by #GtkSourceCompletion::auto-complete-delay.
+ * by the #GtkSourceCompletion:auto-complete-delay property.
  *
  * Returns: the interactive delay in milliseconds.
  **/
