@@ -23,14 +23,9 @@
 #ifndef __GTK_SOURCE_COMPLETION_PROVIDER_H__
 #define __GTK_SOURCE_COMPLETION_PROVIDER_H__
 
-#include <glib.h>
-#include <glib-object.h>
 #include <gtk/gtk.h>
-
-#include <gtksourceview/gtksourcecompletionproposal.h>
-#include <gtksourceview/gtksourcecompletioninfo.h>
-#include <gtksourceview/gtksourcelanguage.h>
 #include <gtksourceview/gtksourcecompletioncontext.h>
+#include <gtksourceview/gtksourcetypes.h>
 
 G_BEGIN_DECLS
 
@@ -39,12 +34,36 @@ G_BEGIN_DECLS
 #define GTK_SOURCE_IS_COMPLETION_PROVIDER(obj) 			(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_SOURCE_TYPE_COMPLETION_PROVIDER))
 #define GTK_SOURCE_COMPLETION_PROVIDER_GET_INTERFACE(obj) 	(G_TYPE_INSTANCE_GET_INTERFACE ((obj), GTK_SOURCE_TYPE_COMPLETION_PROVIDER, GtkSourceCompletionProviderIface))
 
-#define GTK_SOURCE_COMPLETION_CAPABILITY_INTERACTIVE "standard::interactive"
-#define GTK_SOURCE_COMPLETION_CAPABILITY_AUTOMATIC "standard::automatic"
-
-typedef struct _GtkSourceCompletionProvider GtkSourceCompletionProvider;
 typedef struct _GtkSourceCompletionProviderIface GtkSourceCompletionProviderIface;
 
+/**
+ * GtkSourceCompletionProviderIface:
+ * @g_iface: The parent interface.
+ * @get_name: The virtual function pointer for gtk_source_completion_provider_get_name().
+ * Must be implemented.
+ * @get_icon: The virtual function pointer for gtk_source_completion_provider_get_icon().
+ * By default, %NULL is returned.
+ * @populate: The virtual function pointer for gtk_source_completion_provider_populate().
+ * Add no proposals by default.
+ * @match: The virtual function pointer for gtk_source_completion_provider_match().
+ * By default, %TRUE is returned.
+ * @get_activation: The virtual function pointer for gtk_source_completion_provider_get_activation().
+ * The combination of all #GtkSourceCompletionActivation is returned by default.
+ * @get_info_widget: The virtual function pointer for gtk_source_completion_provider_get_info_widget().
+ * By default, %NULL is returned.
+ * @update_info: The virtual function pointer for gtk_source_completion_provider_update_info().
+ * Does nothing by default.
+ * @get_start_iter: The virtual function pointer for gtk_source_completion_provider_get_start_iter().
+ * By default, %FALSE is returned.
+ * @activate_proposal: The virtual function pointer for gtk_source_completion_provider_activate_proposal().
+ * By default, %FALSE is returned.
+ * @get_interactive_delay: The virtual function pointer for gtk_source_completion_provider_get_interactive_delay().
+ * By default, -1 is returned.
+ * @get_priority: The virtual function pointer for gtk_source_completion_provider_get_priority().
+ * By default, 0 is returned.
+ *
+ * The virtual function table for #GtkSourceCompletionProvider.
+ */
 struct _GtkSourceCompletionProviderIface
 {
 	GTypeInterface g_iface;
