@@ -19,6 +19,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "gtksourcegutter.h"
 #include "gtksourcegutter-private.h"
 #include "gtksourceview.h"
 #include "gtksourceview-i18n.h"
@@ -26,8 +27,6 @@
 #include "gtksourcegutterrenderer.h"
 #include "gtksourcegutterrenderer-private.h"
 
-#include <gobject/gvaluecollector.h>
-
 /**
  * SECTION:gutter
  * @Short_description: Gutter object for #GtkSourceView
@@ -37,7 +36,7 @@
  * The #GtkSourceGutter object represents the left and right gutters of the text
  * view. It is used by #GtkSourceView to draw the line numbers and category
  * marks that might be present on a line. By packing additional #GtkSourceGutterRenderer
- * objects in the gutter, you can extend the gutter with your own custom 
+ * objects in the gutter, you can extend the gutter with your own custom
  * drawings.
  *
  * The gutter works very much the same way as cells rendered in a #GtkTreeView.
@@ -49,8 +48,6 @@
  *
  */
 
-#define GTK_SOURCE_GUTTER_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_SOURCE_TYPE_GUTTER, GtkSourceGutterPrivate))
-
 /* Properties */
 enum
 {
@@ -104,7 +101,7 @@ struct _GtkSourceGutterPrivate
 	guint is_drawing : 1;
 };
 
-G_DEFINE_TYPE (GtkSourceGutter, gtk_source_gutter, G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceGutter, gtk_source_gutter, G_TYPE_OBJECT)
 
 static gboolean on_view_draw (GtkSourceView   *view,
                               cairo_t         *cr,
@@ -245,12 +242,6 @@ renderer_free (Renderer *renderer)
 	g_slice_free (Renderer, renderer);
 }
 
-static void
-gtk_source_gutter_finalize (GObject *object)
-{
-	G_OBJECT_CLASS (gtk_source_gutter_parent_class)->finalize (object);
-}
-
 static void
 gtk_source_gutter_dispose (GObject *object)
 {
@@ -532,7 +523,6 @@ gtk_source_gutter_class_init (GtkSourceGutterClass *klass)
 	object_class->set_property = gtk_source_gutter_set_property;
 	object_class->get_property = gtk_source_gutter_get_property;
 
-	object_class->finalize = gtk_source_gutter_finalize;
 	object_class->dispose = gtk_source_gutter_dispose;
 	object_class->constructed = gtk_source_gutter_constructed;
 
@@ -545,6 +535,9 @@ gtk_source_gutter_class_init (GtkSourceGutterClass *klass)
 	                                 PROP_VIEW,
 	                                 g_param_spec_object ("view",
 	                                                      _("View"),
+							      /* Translators: gutters can be used to display information
+							       * on the left or on the right of the text view. To draw the
+							       * line numbers for example. */
 	                                                      _("The gutters' GtkSourceView"),
 	                                                      GTK_SOURCE_TYPE_VIEW,
 	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
@@ -558,14 +551,11 @@ gtk_source_gutter_class_init (GtkSourceGutterClass *klass)
 	                                 PROP_WINDOW_TYPE,
 	                                 g_param_spec_enum ("window_type",
 	                                                    _("Window Type"),
-	                                                    _("The gutters text window type"),
+	                                                    _("The gutters' text window type"),
 	                                                    GTK_TYPE_TEXT_WINDOW_TYPE,
 	                                                    0,
 	                                                    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-	g_type_class_add_private (object_class, sizeof(GtkSourceGutterPrivate));
-
-
 	g_object_class_install_property (object_class,
 	                                 PROP_XPAD,
 	                                 g_param_spec_int ("xpad",
@@ -591,7 +581,7 @@ gtk_source_gutter_class_init (GtkSourceGutterClass *klass)
 static void
 gtk_source_gutter_init (GtkSourceGutter *self)
 {
-	self->priv = GTK_SOURCE_GUTTER_GET_PRIVATE (self);
+	self->priv = gtk_source_gutter_get_instance_private (self);
 }
 
 static gint
@@ -675,7 +665,6 @@ gtk_source_gutter_get_window (GtkSourceGutter *gutter)
  * Since: 3.0
  *
  **/
-
 gboolean
 gtk_source_gutter_insert (GtkSourceGutter         *gutter,
                           GtkSourceGutterRenderer *renderer,
@@ -1027,7 +1016,7 @@ on_view_draw (GtkSourceView   *view,
 	style_context = gtk_widget_get_style_context (GTK_WIDGET (view));
 
 	gtk_style_context_get_color (style_context,
-	                             gtk_widget_get_state (GTK_WIDGET (view)),
+	                             gtk_widget_get_state_flags (GTK_WIDGET (view)),
 	                             &fg_color);
 
 	gdk_cairo_set_source_rgba (cr, &fg_color);
