@@ -25,12 +25,9 @@
 #include "gtksourceview-typebuiltins.h"
 #include "gtksourceview-i18n.h"
 
-#define GTK_SOURCE_GUTTER_RENDERER_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_SOURCE_TYPE_GUTTER_RENDERER, GtkSourceGutterRendererPrivate))
-
 enum
 {
 	ACTIVATE,
-	SIZE_CHANGED,
 	QUEUE_DRAW,
 	QUERY_TOOLTIP,
 	QUERY_DATA,
@@ -62,9 +59,7 @@ struct _GtkSourceGutterRendererPrivate
 
 static guint signals[NUM_SIGNALS] = {0,};
 
-G_DEFINE_ABSTRACT_TYPE (GtkSourceGutterRenderer,
-                        gtk_source_gutter_renderer,
-                        G_TYPE_INITIALLY_UNOWNED)
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GtkSourceGutterRenderer, gtk_source_gutter_renderer, G_TYPE_INITIALLY_UNOWNED)
 
 enum
 {
@@ -83,9 +78,22 @@ enum
 };
 
 static void
-gtk_source_gutter_renderer_finalize (GObject *object)
+set_buffer (GtkSourceGutterRenderer *renderer,
+            GtkTextBuffer           *buffer)
 {
-	G_OBJECT_CLASS (gtk_source_gutter_renderer_parent_class)->finalize (object);
+	if (renderer->priv->buffer != NULL)
+	{
+		g_object_remove_weak_pointer (G_OBJECT (renderer->priv->buffer),
+		                              (gpointer) &renderer->priv->buffer);
+	}
+
+	if (buffer != NULL)
+	{
+		g_object_add_weak_pointer (G_OBJECT (buffer),
+		                           (gpointer) &renderer->priv->buffer);
+	}
+
+	renderer->priv->buffer = buffer;
 }
 
 static void
@@ -104,8 +112,7 @@ emit_buffer_changed (GtkTextView             *view,
 			                                                                renderer->priv->buffer);
 		}
 
-		renderer->priv->buffer = buffer;
-		g_object_add_weak_pointer (G_OBJECT (buffer), (gpointer)&renderer->priv->buffer);
+		set_buffer (renderer, buffer);
 	}
 }
 
@@ -146,6 +153,8 @@ gtk_source_gutter_renderer_dispose (GObject *object)
 
 	renderer = GTK_SOURCE_GUTTER_RENDERER (object);
 
+	set_buffer (renderer, NULL);
+
 	if (renderer->priv->view)
 	{
 		_gtk_source_gutter_renderer_set_view (renderer,
@@ -156,24 +165,21 @@ gtk_source_gutter_renderer_dispose (GObject *object)
 	G_OBJECT_CLASS (gtk_source_gutter_renderer_parent_class)->dispose (object);
 }
 
-static gboolean
+static void
 set_visible (GtkSourceGutterRenderer *renderer,
              gboolean                 visible)
 {
-	if (renderer->priv->visible == visible)
+	visible = visible != FALSE;
+
+	if (renderer->priv->visible != visible)
 	{
-		return FALSE;
+		renderer->priv->visible = visible;
+		g_object_notify (G_OBJECT (renderer), "visible");
+
+		gtk_source_gutter_renderer_queue_draw (renderer);
 	}
-
-	renderer->priv->visible = visible;
-	g_object_notify (G_OBJECT (renderer), "visible");
-
-	gtk_source_gutter_renderer_queue_draw (renderer);
-
-	return TRUE;
 }
 
-
 static gboolean
 set_padding (GtkSourceGutterRenderer *renderer,
              gint                    *field,
@@ -442,7 +448,6 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	object_class->finalize = gtk_source_gutter_renderer_finalize;
 	object_class->dispose = gtk_source_gutter_renderer_dispose;
 
 	object_class->get_property = gtk_source_gutter_renderer_get_property;
@@ -451,8 +456,6 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 	klass->draw = renderer_draw_impl;
 	klass->change_view = renderer_change_view_impl;
 
-	g_type_class_add_private (object_class, sizeof (GtkSourceGutterRendererPrivate));
-
 	/**
 	 * GtkSourceGutterRenderer:visible:
 	 *
@@ -561,10 +564,9 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 	 * @renderer: the #GtkSourceGutterRenderer who emits the signal
 	 *
 	 * The ::queue-draw signal is emitted when the renderer needs
-	 * to be redrawn. Use #gtk_source_gutter_renderer_queue_draw
+	 * to be redrawn. Use gtk_source_gutter_renderer_queue_draw()
 	 * to emit this signal from an implementation of the
 	 * #GtkSourceGutterRenderer interface.
-	 *
 	 */
 	signals[QUEUE_DRAW] =
 		g_signal_new ("queue-draw",
@@ -736,7 +738,7 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 static void
 gtk_source_gutter_renderer_init (GtkSourceGutterRenderer *self)
 {
-	self->priv = GTK_SOURCE_GUTTER_RENDERER_GET_PRIVATE (self);
+	self->priv = gtk_source_gutter_renderer_get_instance_private (self);
 }
 
 /**
@@ -1015,14 +1017,7 @@ gtk_source_gutter_renderer_set_visible (GtkSourceGutterRenderer *renderer,
 {
 	g_return_if_fail (GTK_SOURCE_IS_GUTTER_RENDERER (renderer));
 
-	if (visible != renderer->priv->visible)
-	{
-		renderer->priv->visible = visible;
-
-		g_object_notify (G_OBJECT (renderer), "visible");
-
-		gtk_source_gutter_renderer_queue_draw (renderer);
-	}
+	set_visible (renderer, visible);
 }
 
 /**
@@ -1308,15 +1303,7 @@ _gtk_source_gutter_renderer_set_view (GtkSourceGutterRenderer *renderer,
 	old_view = renderer->priv->view;
 
 	renderer->priv->window_type = window_type;
-
-	if (view)
-	{
-		renderer->priv->view = g_object_ref (view);
-	}
-	else
-	{
-		renderer->priv->view = NULL;
-	}
+	renderer->priv->view = view != NULL ? g_object_ref (view) : NULL;
 
 	if (GTK_SOURCE_GUTTER_RENDERER_GET_CLASS (renderer)->change_view)
 	{
