@@ -8,12 +8,12 @@
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * GtkSourceView is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -21,12 +21,12 @@
 
 #include "gtksourcegutterrenderermarks.h"
 #include "gtksourceview.h"
+#include "gtksourcebuffer.h"
 #include "gtksourcemarkattributes.h"
+#include "gtksourcemark.h"
 
 #define COMPOSITE_ALPHA                 225
 
-#define GTK_SOURCE_GUTTER_RENDERER_MARKS_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_SOURCE_TYPE_GUTTER_RENDERER_MARKS, GtkSourceGutterRendererMarksPrivate))
-
 struct _GtkSourceGutterRendererMarksPrivate
 {
 };
@@ -202,7 +202,6 @@ gutter_renderer_query_data (GtkSourceGutterRenderer *renderer,
 	g_object_set (G_OBJECT (renderer),
 	              "pixbuf", pixbuf,
 	              "xpad", 2,
-	              "ypad", 1,
 	              "yalign", 0.5,
 	              "xalign", 0.5,
 	              "alignment-mode", GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_FIRST,
@@ -418,6 +417,7 @@ gutter_renderer_change_view (GtkSourceGutterRenderer *renderer,
                              GtkTextView             *old_view)
 {
 	GtkSourceView *view;
+	GtkSourceGutterRendererClass *parent_class;
 
 	view = GTK_SOURCE_VIEW (gtk_source_gutter_renderer_get_view (renderer));
 
@@ -426,6 +426,9 @@ gutter_renderer_change_view (GtkSourceGutterRenderer *renderer,
 		gtk_source_gutter_renderer_set_size (renderer,
 		                                     measure_line_height (view));
 	}
+
+	parent_class = GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_marks_parent_class);
+	parent_class->change_view (renderer, old_view);
 }
 
 static void
@@ -440,14 +443,11 @@ gtk_source_gutter_renderer_marks_class_init (GtkSourceGutterRendererMarksClass *
 	renderer_class->query_tooltip = gutter_renderer_query_tooltip;
 	renderer_class->query_activatable = gutter_renderer_query_activatable;
 	renderer_class->change_view = gutter_renderer_change_view;
-
-	/*g_type_class_add_private (object_class, sizeof (GtkSourceGutterRendererMarksPrivate));*/
 }
 
 static void
 gtk_source_gutter_renderer_marks_init (GtkSourceGutterRendererMarks *self)
 {
-	/*self->priv = GTK_SOURCE_GUTTER_RENDERER_MARKS_GET_PRIVATE (self);*/
 }
 
 GtkSourceGutterRenderer *
