@@ -22,6 +22,7 @@
 #ifndef __GTK_SOURCE_GUTTER_RENDERER_TEXT_H__
 #define __GTK_SOURCE_GUTTER_RENDERER_TEXT_H__
 
+#include <gtksourceview/gtksourcetypes.h>
 #include <gtksourceview/gtksourcegutterrenderer.h>
 
 G_BEGIN_DECLS
@@ -34,7 +35,6 @@ G_BEGIN_DECLS
 #define GTK_SOURCE_IS_GUTTER_RENDERER_TEXT_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_SOURCE_TYPE_GUTTER_RENDERER_TEXT))
 #define GTK_SOURCE_GUTTER_RENDERER_TEXT_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_SOURCE_TYPE_GUTTER_RENDERER_TEXT, GtkSourceGutterRendererTextClass))
 
-typedef struct _GtkSourceGutterRendererText		GtkSourceGutterRendererText;
 typedef struct _GtkSourceGutterRendererTextClass	GtkSourceGutterRendererTextClass;
 typedef struct _GtkSourceGutterRendererTextPrivate	GtkSourceGutterRendererTextPrivate;
 
