@@ -20,6 +20,7 @@
  */
 
 #include "gtksourcestyleschememanager.h"
+#include "gtksourcestylescheme.h"
 #include "gtksourceview-marshal.h"
 #include "gtksourceview-i18n.h"
 #include "gtksourceview-utils.h"
@@ -30,6 +31,8 @@
  * @Short_description: Object which provides access to #GtkSourceStyleScheme<!-- -->s
  * @Title: GtkSourceStyleSchemeManager
  * @See_also: #GtkSourceStyleScheme
+ *
+ * Object which provides access to #GtkSourceStyleScheme<!-- -->s.
  */
 
 #define SCHEME_FILE_SUFFIX	".xml"
@@ -45,15 +48,13 @@ struct _GtkSourceStyleSchemeManagerPrivate
 	gchar          **ids; /* Cache the IDs of the available schemes */
 };
 
-
 enum {
 	PROP_0,
 	PROP_SEARCH_PATH,
 	PROP_SCHEME_IDS
 };
 
-
-G_DEFINE_TYPE (GtkSourceStyleSchemeManager, gtk_source_style_scheme_manager, G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceStyleSchemeManager, gtk_source_style_scheme_manager, G_TYPE_OBJECT)
 
 static void
 gtk_source_style_scheme_manager_set_property (GObject 	   *object,
@@ -163,16 +164,12 @@ gtk_source_style_scheme_manager_class_init (GtkSourceStyleSchemeManagerClass *kl
 							       "style schemes"),
 							     G_TYPE_STRV,
 							     G_PARAM_READABLE));
-
-	g_type_class_add_private (object_class, sizeof(GtkSourceStyleSchemeManagerPrivate));
 }
 
 static void
 gtk_source_style_scheme_manager_init (GtkSourceStyleSchemeManager *mgr)
 {
-	mgr->priv = G_TYPE_INSTANCE_GET_PRIVATE (mgr,
-						 GTK_SOURCE_TYPE_STYLE_SCHEME_MANAGER,
-						 GtkSourceStyleSchemeManagerPrivate);
+	mgr->priv = gtk_source_style_scheme_manager_get_instance_private (mgr);
 	mgr->priv->schemes_hash = NULL;
 	mgr->priv->ids = NULL;
 	mgr->priv->search_path = NULL;
@@ -362,12 +359,12 @@ reload_if_needed (GtkSourceStyleSchemeManager *mgr)
 		if (scheme != NULL)
 		{
 			const gchar *id = gtk_source_style_scheme_get_id (scheme);
-			GtkSourceStyleScheme *old;
 
-			old = g_hash_table_lookup (schemes_hash, id);
-			if (old != NULL)
+			/* scheme with the same id already loaded from a path with higher prio: skip it */
+			if (g_hash_table_contains (schemes_hash, id))
 			{
-				schemes = g_slist_remove (schemes, old);
+				g_object_unref (scheme);
+				continue;
 			}
 
 			schemes = g_slist_prepend (schemes, scheme);
