@@ -25,11 +25,7 @@
 #define __GTK_SOURCE_VIEW_H__
 
 #include <gtk/gtk.h>
-
-#include <gtksourceview/gtksourcebuffer.h>
-#include <gtksourceview/gtksourcecompletion.h>
-#include <gtksourceview/gtksourcegutter.h>
-#include <gtksourceview/gtksourcemarkattributes.h>
+#include <gtksourceview/gtksourcetypes.h>
 
 G_BEGIN_DECLS
 
@@ -53,7 +49,6 @@ typedef enum
 	GTK_SOURCE_VIEW_GUTTER_POSITION_MARKS = -20
 } GtkSourceViewGutterPosition;
 
-typedef struct _GtkSourceView GtkSourceView;
 typedef struct _GtkSourceViewClass GtkSourceViewClass;
 
 typedef struct _GtkSourceViewPrivate GtkSourceViewPrivate;
@@ -130,13 +125,7 @@ typedef enum
 	GTK_SOURCE_DRAW_SPACES_LEADING    = 1 << 4,
 	GTK_SOURCE_DRAW_SPACES_TEXT       = 1 << 5,
 	GTK_SOURCE_DRAW_SPACES_TRAILING   = 1 << 6,
-	GTK_SOURCE_DRAW_SPACES_ALL        = (GTK_SOURCE_DRAW_SPACES_SPACE   | \
-	                                     GTK_SOURCE_DRAW_SPACES_TAB     | \
-	                                     GTK_SOURCE_DRAW_SPACES_NEWLINE | \
-	                                     GTK_SOURCE_DRAW_SPACES_NBSP | \
-	                                     GTK_SOURCE_DRAW_SPACES_LEADING | \
-	                                     GTK_SOURCE_DRAW_SPACES_TEXT | \
-	                                     GTK_SOURCE_DRAW_SPACES_TRAILING)
+	GTK_SOURCE_DRAW_SPACES_ALL        = 0x7f
 } GtkSourceDrawSpacesFlags;
 
 
