@@ -20,35 +20,16 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/* If TEST_XML_MEM is defined the test program will try to detect memory
- * allocated by xmlMalloc() but not freed by xmlFree() or freed by xmlFree()
- * but not allocated by xmlMalloc(). */
-#define TEST_XML_MEM
-
-#include <stdio.h>
 #include <string.h>
-#include <stdlib.h>
 #include <gtk/gtk.h>
 #include <gio/gio.h>
-#include <gtksourceview/gtksourceview.h>
-#include <gtksourceview/gtksourcelanguage.h>
-#include <gtksourceview/gtksourcelanguagemanager.h>
-#include <gtksourceview/gtksourcestyleschememanager.h>
-#include <gtksourceview/gtksourceprintcompositor.h>
-#include <gtksourceview/gtksourceview-typebuiltins.h>
-#ifdef TEST_XML_MEM
-#include <libxml/xmlreader.h>
-#endif
+#include <gtksourceview/gtksource.h>
 
 /* Global list of open windows */
-
 static GList *windows = NULL;
+
 static GtkSourceStyleScheme *style_scheme = NULL;
 
-/* Private data structures */
-
-#define READ_BUFFER_SIZE   4096
-
 #define MARK_TYPE_1      "one"
 #define MARK_TYPE_2      "two"
 
@@ -59,10 +40,6 @@ static void       open_file_cb                   (GtkAction       *action,
 						  gpointer         user_data);
 static void       print_file_cb                  (GtkAction       *action,
 						  gpointer         user_data);
-static void       find_cb			 (GtkAction       *action,
-						  gpointer         user_data);
-static void       replace_cb			 (GtkAction       *action,
-						  gpointer         user_data);
 
 static void       new_view_cb                    (GtkAction       *action,
 						  gpointer         user_data);
@@ -123,10 +100,6 @@ static GtkActionEntry view_action_entries[] = {
 	{ "TabWidth", NULL, "_Tab Width", NULL, NULL, NULL },
 	{ "IndentWidth", NULL, "I_ndent Width", NULL, NULL, NULL },
 	{ "SmartHomeEnd", NULL, "_Smart Home/End", NULL, NULL, NULL },
-	{ "Find", GTK_STOCK_FIND, "_Find", "<control>F",
-	  "Find", G_CALLBACK (find_cb) },
-	{ "Replace", GTK_STOCK_FIND_AND_REPLACE, "Search and _Replace", "<control>R",
-	  "Search and Replace", G_CALLBACK (replace_cb) },
 	{ "ForwardString", NULL, "_Forward to string toggle", "<control>S",
 	  "Forward to the start or end of the next string", G_CALLBACK (forward_string_cb) },
 	{ "BackwardString", NULL, "_Backward to string toggle", "<control><shift>S",
@@ -252,8 +225,6 @@ static const gchar *buffer_ui_description =
 "    <menu action=\"FileMenu\">"
 "      <menuitem action=\"Open\"/>"
 "      <menuitem action=\"Print\"/>"
-"      <menuitem action=\"Find\"/>"
-"      <menuitem action=\"Replace\"/>"
 "      <separator/>"
 "      <menuitem action=\"Quit\"/>"
 "    </menu>"
@@ -265,100 +236,79 @@ static const gchar *buffer_ui_description =
 
 /* File loading code ----------------------------------------------------------------- */
 
-static void
-error_dialog (GtkWindow *parent, const gchar *msg, ...)
-{
-	va_list ap;
-	gchar *tmp;
-	GtkWidget *dialog;
-
-	va_start (ap, msg);
-	tmp = g_strdup_vprintf (msg, ap);
-	va_end (ap);
-
-	dialog = gtk_message_dialog_new (parent,
-					 GTK_DIALOG_DESTROY_WITH_PARENT,
-					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_OK,
-					 "%s", tmp);
-	g_free (tmp);
-
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
-}
-
 static gboolean
-gtk_source_buffer_load_file (GtkSourceBuffer *source_buffer,
-			     const gchar     *filename,
-			     GError         **error)
+gtk_source_buffer_load_file (GtkSourceBuffer *buffer,
+			     const gchar     *filename)
 {
 	GtkTextIter iter;
-	gchar *buffer;
-	GError *error_here = NULL;
+	gchar *contents;
+	GError *error = NULL;
 
-	g_return_val_if_fail (GTK_SOURCE_IS_BUFFER (source_buffer), FALSE);
+	g_return_val_if_fail (GTK_SOURCE_IS_BUFFER (buffer), FALSE);
 	g_return_val_if_fail (filename != NULL, FALSE);
 
-	if (!g_file_get_contents (filename, &buffer, NULL, &error_here))
+	if (!g_file_get_contents (filename, &contents, NULL, &error))
 	{
-		error_dialog (NULL, "%s\nFile %s", error_here->message, filename);
-		g_propagate_error (error, error_here);
+		GtkWidget *dialog = gtk_message_dialog_new (NULL,
+							    0,
+							    GTK_MESSAGE_ERROR,
+							    GTK_BUTTONS_OK,
+							    "%s\nFile %s",
+							    error->message,
+							    filename);
+
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+
+		g_error_free (error);
 		return FALSE;
 	}
 
-	gtk_source_buffer_begin_not_undoable_action (source_buffer);
-	gtk_text_buffer_set_text (GTK_TEXT_BUFFER (source_buffer), buffer, -1);
-	gtk_source_buffer_end_not_undoable_action (source_buffer);
-	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (source_buffer), FALSE);
+	gtk_source_buffer_begin_not_undoable_action (buffer);
+	gtk_text_buffer_set_text (GTK_TEXT_BUFFER (buffer), contents, -1);
+	gtk_source_buffer_end_not_undoable_action (buffer);
+	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (buffer), FALSE);
 
 	/* move cursor to the beginning */
-	gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (source_buffer), &iter);
-	gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (source_buffer), &iter);
+	gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (buffer), &iter);
+	gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (buffer), &iter);
 
-	{
-		GtkTextIter start, end;
-		char *text;
-		gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (source_buffer), &start, &end);
-		text = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (source_buffer), &start, &end, TRUE);
-		g_assert (!strcmp (text, buffer));
-		g_free (text);
-	}
-
-	g_free (buffer);
+	g_free (contents);
 	return TRUE;
 }
 
 static void
 remove_all_marks (GtkSourceBuffer *buffer)
 {
-	GtkTextIter s, e;
+	GtkTextIter start;
+	GtkTextIter end;
 
-	gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (buffer), &s, &e);
+	gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (buffer), &start, &end);
 
-	gtk_source_buffer_remove_source_marks (buffer, &s, &e, NULL);
+	gtk_source_buffer_remove_source_marks (buffer, &start, &end, NULL);
 }
 
 static GtkSourceLanguage *
-get_language_for_file (GtkTextBuffer *buffer, const gchar *filename)
+get_language_for_file (GtkTextBuffer *buffer,
+		       const gchar   *filename)
 {
 	GtkSourceLanguageManager *manager;
 	GtkSourceLanguage *language;
-	GtkTextIter start, end;
+	GtkTextIter start;
+	GtkTextIter end;
 	gchar *text;
 	gchar *content_type;
 	gboolean result_uncertain;
 
 	gtk_text_buffer_get_start_iter (buffer, &start);
-	if (gtk_text_buffer_get_char_count (buffer) < 1024)
-		gtk_text_buffer_get_end_iter (buffer, &end);
-	else
-		gtk_text_buffer_get_iter_at_offset (buffer, &end, 1024);
+	gtk_text_buffer_get_iter_at_offset (buffer, &end, 1024);
 	text = gtk_text_buffer_get_slice (buffer, &start, &end, TRUE);
 
 	content_type = g_content_type_guess (filename,
 					     (guchar*) text,
 					     strlen (text),
 					     &result_uncertain);
+
 	if (result_uncertain)
 	{
 		g_free (content_type);
@@ -371,9 +321,9 @@ get_language_for_file (GtkTextBuffer *buffer, const gchar *filename)
 							       content_type);
 
 	g_message ("Detected '%s' mime type for file %s, chose language %s",
-		   content_type ? content_type : "(null)",
+		   content_type != NULL ? content_type : "(null)",
 		   filename,
-		   language ? gtk_source_language_get_id (language) : "(none)");
+		   language != NULL ? gtk_source_language_get_id (language) : "(none)");
 
 	g_free (content_type);
 	g_free (text);
@@ -389,10 +339,12 @@ get_language_by_id (const gchar *id)
 }
 
 static GtkSourceLanguage *
-get_language (GtkTextBuffer *buffer, const gchar *filename)
+get_language (GtkTextBuffer *buffer,
+	      const gchar   *filename)
 {
 	GtkSourceLanguage *language = NULL;
-	GtkTextIter start, end;
+	GtkTextIter start;
+	GtkTextIter end;
 	gchar *text;
 	gchar *lang_string;
 
@@ -401,8 +353,10 @@ get_language (GtkTextBuffer *buffer, const gchar *filename)
 	gtk_text_iter_forward_line (&end);
 
 #define LANG_STRING "gtk-source-lang:"
+
 	text = gtk_text_iter_get_slice (&start, &end);
 	lang_string = strstr (text, LANG_STRING);
+
 	if (lang_string != NULL)
 	{
 		gchar **tokens;
@@ -413,91 +367,104 @@ get_language (GtkTextBuffer *buffer, const gchar *filename)
 		tokens = g_strsplit_set (lang_string, " \t\n", 2);
 
 		if (tokens != NULL && tokens[0] != NULL)
+		{
 			language = get_language_by_id (tokens[0]);
+		}
 
 		g_strfreev (tokens);
 	}
 
-	if (!language)
+	if (language == NULL)
+	{
 		language = get_language_for_file (buffer, filename);
+	}
 
 	g_free (text);
 	return language;
 }
 
-static gboolean
-open_file (GtkSourceBuffer *buffer, const gchar *filename)
+static void
+print_language_style_ids (GtkSourceLanguage *language)
+{
+	gchar **styles;
+
+	g_assert (language != NULL);
+
+	styles = gtk_source_language_get_style_ids (language);
+
+	if (styles == NULL)
+	{
+		g_print ("No styles in language '%s'\n",
+			 gtk_source_language_get_name (language));
+	}
+	else
+	{
+		gchar **ids;
+		g_print ("Styles in language '%s':\n",
+			 gtk_source_language_get_name (language));
+
+		for (ids = styles; *ids != NULL; ids++)
+		{
+			const gchar *name = gtk_source_language_get_style_name (language, *ids);
+
+			g_print ("- %s (name: '%s')\n", *ids, name);
+		}
+
+		g_strfreev (styles);
+	}
+
+	g_print ("\n");
+}
+
+static void
+open_file (GtkSourceBuffer *buffer,
+	   const gchar     *filename)
 {
 	GtkSourceLanguage *language = NULL;
-	gchar *freeme = NULL;
-	gboolean success = FALSE;
+	gchar *absolute_filename;
 
-	if (!g_path_is_absolute (filename))
+	if (g_path_is_absolute (filename))
+	{
+		absolute_filename = g_strdup (filename);
+	}
+	else
 	{
 		gchar *curdir = g_get_current_dir ();
-		freeme = g_build_filename (curdir, filename, NULL);
-		filename = freeme;
+		absolute_filename = g_build_filename (curdir, filename, NULL);
 		g_free (curdir);
 	}
 
 	remove_all_marks (buffer);
 
-	success = gtk_source_buffer_load_file (buffer, filename, NULL);
+	if (!gtk_source_buffer_load_file (buffer, absolute_filename))
+	{
+		g_free (absolute_filename);
+		return;
+	}
 
-	if (!success)
-		goto out;
-
-	language = get_language (GTK_TEXT_BUFFER (buffer), filename);
-
-	if (language == NULL)
-		g_print ("No language found for file `%s'\n", filename);
-
-	gtk_source_buffer_set_language (buffer, language);
 	g_object_set_data_full (G_OBJECT (buffer),
-				"filename", g_strdup (filename),
+				"filename", absolute_filename,
 				(GDestroyNotify) g_free);
 
+	language = get_language (GTK_TEXT_BUFFER (buffer), absolute_filename);
+	gtk_source_buffer_set_language (buffer, language);
+
 	if (language != NULL)
 	{
-		gchar **styles;
-
-		styles = gtk_source_language_get_style_ids (language);
-
-		if (styles == NULL)
-			g_print ("No styles in language '%s'\n", gtk_source_language_get_name (language));
-		else
-		{
-			gchar **ids;
-			g_print ("Styles in in language '%s':\n", gtk_source_language_get_name (language));
-
-			ids = styles;
-
-			while (*ids != NULL)
-			{
-				const gchar *name;
-
-				name = gtk_source_language_get_style_name (language, *ids);
-
-				g_print ("- %s (name: '%s')\n", *ids, name);
-
-				++ids;
-			}
-
-			g_strfreev (styles);
-		}
-
-		g_print("\n");
+		print_language_style_ids (language);
+	}
+	else
+	{
+		g_print ("No language found for file '%s'\n", absolute_filename);
 	}
-out:
-	g_free (freeme);
-	return success;
 }
 
 
 /* View action callbacks -------------------------------------------------------- */
 
 static void
-numbers_toggled_cb (GtkAction *action, gpointer user_data)
+numbers_toggled_cb (GtkAction *action,
+		    gpointer   user_data)
 {
 	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action) && GTK_SOURCE_IS_VIEW (user_data));
 	gtk_source_view_set_show_line_numbers (
@@ -506,7 +473,8 @@ numbers_toggled_cb (GtkAction *action, gpointer user_data)
 }
 
 static void
-marks_toggled_cb (GtkAction *action, gpointer user_data)
+marks_toggled_cb (GtkAction *action,
+		  gpointer   user_data)
 {
 	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action) && GTK_SOURCE_IS_VIEW (user_data));
 	gtk_source_view_set_show_line_marks (
@@ -515,7 +483,8 @@ marks_toggled_cb (GtkAction *action, gpointer user_data)
 }
 
 static void
-margin_toggled_cb (GtkAction *action, gpointer user_data)
+margin_toggled_cb (GtkAction *action,
+		   gpointer   user_data)
 {
 	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action) && GTK_SOURCE_IS_VIEW (user_data));
 	gtk_source_view_set_show_right_margin (
@@ -524,7 +493,8 @@ margin_toggled_cb (GtkAction *action, gpointer user_data)
 }
 
 static void
-hl_syntax_toggled_cb (GtkAction *action, gpointer user_data)
+hl_syntax_toggled_cb (GtkAction *action,
+		      gpointer   user_data)
 {
 	GtkTextBuffer *buffer;
 	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action) && GTK_SOURCE_IS_VIEW (user_data));
@@ -535,7 +505,8 @@ hl_syntax_toggled_cb (GtkAction *action, gpointer user_data)
 }
 
 static void
-hl_bracket_toggled_cb (GtkAction *action, gpointer user_data)
+hl_bracket_toggled_cb (GtkAction *action,
+		       gpointer   user_data)
 {
 	GtkTextBuffer *buffer;
 	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action) && GTK_SOURCE_IS_VIEW (user_data));
@@ -546,7 +517,8 @@ hl_bracket_toggled_cb (GtkAction *action, gpointer user_data)
 }
 
 static void
-hl_line_toggled_cb (GtkAction *action, gpointer user_data)
+hl_line_toggled_cb (GtkAction *action,
+		    gpointer   user_data)
 {
 	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action) && GTK_SOURCE_IS_VIEW (user_data));
 	gtk_source_view_set_highlight_current_line (
@@ -555,7 +527,8 @@ hl_line_toggled_cb (GtkAction *action, gpointer user_data)
 }
 
 static void
-draw_spaces_toggled_cb (GtkAction *action, gpointer user_data)
+draw_spaces_toggled_cb (GtkAction *action,
+		       	gpointer   user_data)
 {
 	gboolean draw_spaces;
 
@@ -563,15 +536,20 @@ draw_spaces_toggled_cb (GtkAction *action, gpointer user_data)
 	draw_spaces = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
 
 	if (draw_spaces)
+	{
 		gtk_source_view_set_draw_spaces (GTK_SOURCE_VIEW (user_data),
 						 GTK_SOURCE_DRAW_SPACES_ALL);
+	}
 	else
+	{
 		gtk_source_view_set_draw_spaces (GTK_SOURCE_VIEW (user_data),
 						 0);
+	}
 }
 
 static void
-wrap_lines_toggled_cb (GtkAction *action, gpointer user_data)
+wrap_lines_toggled_cb (GtkAction *action,
+		       gpointer   user_data)
 {
 	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action) && GTK_SOURCE_IS_VIEW (user_data));
 	gtk_text_view_set_wrap_mode (user_data,
@@ -602,7 +580,7 @@ insert_spaces_toggled_cb (GtkAction *action,
 static void
 tabs_toggled_cb (GtkAction *action,
 		 GtkAction *current,
-		 gpointer user_data)
+		 gpointer   user_data)
 {
 	g_return_if_fail (GTK_IS_RADIO_ACTION (action) && GTK_SOURCE_IS_VIEW (user_data));
 	gtk_source_view_set_tab_width (
@@ -613,7 +591,7 @@ tabs_toggled_cb (GtkAction *action,
 static void
 indent_toggled_cb (GtkAction *action,
 		   GtkAction *current,
-		   gpointer user_data)
+		   gpointer   user_data)
 {
 	g_return_if_fail (GTK_IS_RADIO_ACTION (action) && GTK_SOURCE_IS_VIEW (user_data));
 	gtk_source_view_set_indent_width (
@@ -624,7 +602,7 @@ indent_toggled_cb (GtkAction *action,
 static void
 smart_home_end_toggled_cb (GtkAction *action,
 			   GtkAction *current,
-			   gpointer user_data)
+			   gpointer   user_data)
 {
 	g_return_if_fail (GTK_IS_RADIO_ACTION (action) && GTK_SOURCE_IS_VIEW (user_data));
 	gtk_source_view_set_smart_home_end (
@@ -633,7 +611,8 @@ smart_home_end_toggled_cb (GtkAction *action,
 }
 
 static void
-new_view_cb (GtkAction *action, gpointer user_data)
+new_view_cb (GtkAction *action,
+	     gpointer   user_data)
 {
 	GtkSourceBuffer *buffer;
 	GtkSourceView *view;
@@ -707,152 +686,9 @@ backward_string_cb (GtkAction *action,
 
 /* Buffer action callbacks ------------------------------------------------------------ */
 
-static struct {
-	char *what;
-	char *replacement;
-	GtkTextSearchFlags flags;
-} search_data = {
-	NULL,
-	NULL,
-	GTK_TEXT_SEARCH_CASE_INSENSITIVE
-};
-
-static gboolean
-search_dialog (GtkWidget            *widget,
-	       gboolean              replace,
-	       char                **what_p,
-	       char                **replacement_p,
-	       GtkTextSearchFlags   *flags_p)
-{
-	GtkWidget *dialog;
-	GtkEntry *entry1, *entry2;
-	GtkToggleButton *case_sensitive;
-
-	dialog = gtk_dialog_new_with_buttons (replace ? "Replace" : "Find",
-					      GTK_WINDOW (gtk_widget_get_toplevel (widget)),
-					      GTK_DIALOG_MODAL,
-					      GTK_STOCK_CANCEL,
-					      GTK_RESPONSE_CANCEL,
-					      GTK_STOCK_OK,
-					      GTK_RESPONSE_OK,
-					      NULL);
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-
-	entry1 = g_object_new (GTK_TYPE_ENTRY,
-			       "visible", TRUE,
-			       "text", search_data.what ? search_data.what : "",
-			       "activates-default", TRUE,
-			       NULL);
-	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-				     GTK_WIDGET (entry1), TRUE, TRUE, 0);
-	entry2 = g_object_new (GTK_TYPE_ENTRY,
-			       "visible", replace,
-			       "text", search_data.replacement ? search_data.replacement : "",
-			       "activates-default", TRUE,
-			       NULL);
-	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-				     GTK_WIDGET (entry2), TRUE, TRUE, 0);
-
-	case_sensitive = g_object_new (GTK_TYPE_CHECK_BUTTON,
-				       "visible", TRUE,
-				       "label", "Case sensitive",
-				       "active", !(search_data.flags & GTK_TEXT_SEARCH_CASE_INSENSITIVE),
-				       NULL);
-	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-			    GTK_WIDGET (case_sensitive), FALSE, FALSE, 0);
-
-	while (TRUE)
-	{
-		if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_OK)
-		{
-			gtk_widget_destroy (dialog);
-			return FALSE;
-		}
-
-		if (*gtk_entry_get_text (entry1))
-			break;
-	}
-
-	g_free (search_data.what);
-	*what_p = search_data.what = g_strdup (gtk_entry_get_text (entry1));
-	g_free (search_data.replacement);
-	*replacement_p = search_data.replacement = g_strdup (gtk_entry_get_text (entry2));
-	*flags_p = search_data.flags = gtk_toggle_button_get_active (case_sensitive) ?
-					0 : GTK_TEXT_SEARCH_CASE_INSENSITIVE;
-
-	gtk_widget_destroy (dialog);
-	return TRUE;
-}
-
 static void
-do_search_replace (GtkTextView *view,
-		   gboolean     replace)
-{
-	GtkTextBuffer *buffer = gtk_text_view_get_buffer (view);
-	GtkTextIter iter;
-	char *what, *replacement;
-	GtkTextSearchFlags flags;
-
-	if (!search_dialog (GTK_WIDGET (view), replace, &what, &replacement, &flags))
-		return;
-
-	if (replace)
-	{
-		gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
-
-		while (TRUE)
-		{
-			GtkTextIter match_start, match_end;
-
-			if (!gtk_text_iter_forward_search (&iter, what, flags,
-							   &match_start,
-							   &match_end,
-							   NULL))
-			{
-				break;
-			}
-
-			gtk_text_buffer_delete (buffer, &match_start, &match_end);
-			gtk_text_buffer_insert (buffer, &match_start, replacement, -1);
-			iter = match_start;
-		}
-	}
-	else
-	{
-		GtkTextIter match_start, match_end;
-
-		gtk_text_buffer_get_iter_at_mark (buffer, &iter, gtk_text_buffer_get_insert (buffer));
-
-		if (gtk_text_iter_forward_search (&iter, what, flags, &match_start, &match_end, NULL))
-		{
-			gtk_text_buffer_select_range (buffer, &match_start, &match_end);
-		}
-		else
-		{
-			GtkTextIter insert = iter;
-			gtk_text_buffer_get_start_iter (buffer, &iter);
-			if (gtk_text_iter_forward_search (&iter, what, flags, &match_start, &match_end, &insert))
-				gtk_text_buffer_select_range (buffer, &match_start, &match_end);
-		}
-	}
-}
-
-static void
-find_cb (GtkAction *action,
-	 gpointer   user_data)
-{
-	do_search_replace (user_data, FALSE);
-}
-
-static void
-replace_cb (GtkAction *action,
-	    gpointer   user_data)
-{
-	do_search_replace (user_data, TRUE);
-}
-
-static void
-open_file_cb (GtkAction *action, gpointer user_data)
+open_file_cb (GtkAction *action,
+	      gpointer   user_data)
 {
 	GtkWidget *chooser;
 	gint response;
@@ -868,11 +704,15 @@ open_file_cb (GtkAction *action, gpointer user_data)
 					       NULL);
 
 	if (last_dir == NULL)
+	{
 		last_dir = g_strdup (TOP_SRCDIR "/gtksourceview");
+	}
 
 	if (last_dir != NULL && g_path_is_absolute (last_dir))
+	{
 		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
 						     last_dir);
+	}
 
 	response = gtk_dialog_run (GTK_DIALOG (chooser));
 
@@ -931,8 +771,6 @@ paginate (GtkPrintOperation        *operation,
 		n_pages = gtk_source_print_compositor_get_n_pages (compositor);
 		gtk_print_operation_set_n_pages (operation, n_pages);
 
-
-
 		return TRUE;
 	}
 
@@ -1019,7 +857,8 @@ end_print (GtkPrintOperation        *operation,
 
 
 static void
-print_file_cb (GtkAction *action, gpointer user_data)
+print_file_cb (GtkAction *action,
+	       gpointer   user_data)
 {
 	GtkSourceView *view;
 	GtkSourceBuffer *buffer;
@@ -1108,22 +947,18 @@ print_file_cb (GtkAction *action, gpointer user_data)
 /* View UI callbacks ------------------------------------------------------------------ */
 
 static void
-update_cursor_position (GtkTextBuffer *buffer, gpointer user_data)
+update_cursor_position_info (GtkTextBuffer *buffer,
+			     GtkSourceView *view)
 {
 	gchar *msg;
-	gint row, chars;
+	gint row;
+	gint chars;
 	guint col;
 	GtkTextIter iter;
-	GtkSourceView *view;
 	GtkLabel *pos_label;
 	gchar **classes;
 	gchar **classes_ptr;
-	GString *str;
-
-	g_return_if_fail (GTK_SOURCE_IS_VIEW (user_data));
-
-	view = GTK_SOURCE_VIEW (user_data);
-	pos_label = GTK_LABEL (g_object_get_data (G_OBJECT (view), "pos_label"));
+	GString *classes_str;
 
 	gtk_text_buffer_get_iter_at_mark (buffer,
 					  &iter,
@@ -1136,46 +971,50 @@ update_cursor_position (GtkTextBuffer *buffer, gpointer user_data)
 	classes = gtk_source_buffer_get_context_classes_at_iter (GTK_SOURCE_BUFFER (buffer),
 	                                                         &iter);
 
-	str = g_string_new ("");
-	classes_ptr = classes;
+	classes_str = g_string_new ("");
 
-	while (classes_ptr && *classes_ptr)
+	for (classes_ptr = classes; classes_ptr != NULL && *classes_ptr != NULL; classes_ptr++)
 	{
 		if (classes_ptr != classes)
 		{
-			g_string_append (str, ", ");
+			g_string_append (classes_str, ", ");
 		}
 
-		g_string_append_printf (str, "%s", *classes_ptr);
-		++classes_ptr;
+		g_string_append_printf (classes_str, "%s", *classes_ptr);
 	}
 
 	g_strfreev (classes);
 
-	msg = g_strdup_printf ("char: %d, line: %d, column: %d, classes: %s", chars, row, col, str->str);
+	msg = g_strdup_printf ("char: %d, line: %d, column: %u, classes: %s",
+			       chars,
+			       row,
+			       col,
+			       classes_str->str);
+
+	pos_label = GTK_LABEL (g_object_get_data (G_OBJECT (view), "pos_label"));
 	gtk_label_set_text (pos_label, msg);
 
 	g_free (msg);
-	g_string_free (str, TRUE);
+	g_string_free (classes_str, TRUE);
 }
 
 static void
-move_cursor_cb (GtkTextBuffer *buffer,
-		GtkTextIter   *cursoriter,
-		GtkTextMark   *mark,
-		gpointer       user_data)
+mark_set_cb (GtkTextBuffer *buffer,
+	     GtkTextIter   *cursoriter,
+	     GtkTextMark   *mark,
+	     GtkSourceView *view)
 {
-	if (mark != gtk_text_buffer_get_insert (buffer))
-		return;
-
-	update_cursor_position (buffer, user_data);
+	if (mark == gtk_text_buffer_get_insert (buffer))
+	{
+		update_cursor_position_info (buffer, view);
+	}
 }
 
 static gboolean
-window_deleted_cb (GtkWidget *widget, GdkEvent *ev, gpointer user_data)
+window_deleted_cb (GtkWidget     *widget,
+		   GdkEvent      *event,
+		   GtkSourceView *view)
 {
-	g_return_val_if_fail (GTK_SOURCE_IS_VIEW (user_data), TRUE);
-
 	if (g_list_nth_data (windows, 0) == widget)
 	{
 		/* Main (first in the list) window was closed, so exit
@@ -1184,7 +1023,6 @@ window_deleted_cb (GtkWidget *widget, GdkEvent *ev, gpointer user_data)
 	}
 	else
 	{
-		GtkSourceView *view = GTK_SOURCE_VIEW (user_data);
 		GtkSourceBuffer *buffer = GTK_SOURCE_BUFFER (
 			gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
 
@@ -1197,7 +1035,7 @@ window_deleted_cb (GtkWidget *widget, GdkEvent *ev, gpointer user_data)
 						      0, /* detail */
 						      NULL, /* closure */
 						      NULL, /* func */
-						      user_data);
+						      view);
 
 		/* we return FALSE since we want the window destroyed */
 		return FALSE;
@@ -1207,23 +1045,16 @@ window_deleted_cb (GtkWidget *widget, GdkEvent *ev, gpointer user_data)
 }
 
 static void
-line_mark_activated (GtkSourceGutter *gutter, 
+line_mark_activated (GtkSourceGutter *gutter,
                      GtkTextIter     *iter,
-                     GdkEventButton  *ev,
+                     GdkEventButton  *event,
                      GtkSourceView   *view)
 {
 	GtkSourceBuffer *buffer;
 	GSList *mark_list;
 	const gchar *mark_type;
 
-	if (ev->button == 1)
-	{
-		mark_type = MARK_TYPE_1;
-	}
-	else
-	{
-		mark_type = MARK_TYPE_2;
-	}
+	mark_type = event->button == 1 ? MARK_TYPE_1 : MARK_TYPE_2;
 
 	buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
 
@@ -1251,10 +1082,9 @@ line_mark_activated (GtkSourceGutter *gutter,
 }
 
 static void
-bracket_matched (GtkSourceBuffer           *buffer G_GNUC_UNUSED,
+bracket_matched (GtkSourceBuffer           *buffer,
                  GtkTextIter               *iter,
-                 GtkSourceBracketMatchType  state,
-                 gpointer                  *data G_GNUC_UNUSED)
+                 GtkSourceBracketMatchType  state)
 {
 	GEnumClass *eclass;
 	GEnumValue *evalue;
@@ -1284,13 +1114,14 @@ mark_tooltip_func (GtkSourceMarkAttributes *attrs,
                    GtkSourceMark           *mark,
                    GtkSourceView           *view)
 {
-	GtkTextBuffer *buf;
+	GtkTextBuffer *buffer;
 	GtkTextIter iter;
-	gint line, column;
+	gint line;
+	gint column;
 
-	buf = gtk_text_mark_get_buffer (GTK_TEXT_MARK (mark));
+	buffer = gtk_text_mark_get_buffer (GTK_TEXT_MARK (mark));
 
-	gtk_text_buffer_get_iter_at_mark (buf, &iter, GTK_TEXT_MARK (mark));
+	gtk_text_buffer_get_iter_at_mark (buffer, &iter, GTK_TEXT_MARK (mark));
 	line = gtk_text_iter_get_line (&iter) + 1;
 	column = gtk_text_iter_get_line_offset (&iter);
 
@@ -1305,7 +1136,7 @@ mark_tooltip_func (GtkSourceMarkAttributes *attrs,
 }
 
 static void
-add_source_mark_pixbufs (GtkSourceView *view)
+add_source_mark_attributes (GtkSourceView *view)
 {
 	GdkRGBA color;
 	GtkSourceMarkAttributes *attrs;
@@ -1314,12 +1145,17 @@ add_source_mark_pixbufs (GtkSourceView *view)
 
 	attrs = gtk_source_mark_attributes_new ();
 	gtk_source_mark_attributes_set_background (attrs, &color);
+
+	/* FIXME: GTK_STOCK_YES is deprecated, but doesn't have a replacement as
+	 * a named icon.
+	 */
 	gtk_source_mark_attributes_set_stock_id (attrs, GTK_STOCK_YES);
 
 	g_signal_connect (attrs,
 	                  "query-tooltip-markup",
 	                  G_CALLBACK (mark_tooltip_func),
 	                  view);
+
 	gtk_source_view_set_mark_attributes (view, MARK_TYPE_1, attrs, 1);
 
 	gdk_rgba_parse (&color, "pink");
@@ -1327,6 +1163,8 @@ add_source_mark_pixbufs (GtkSourceView *view)
 	attrs = gtk_source_mark_attributes_new ();
 
 	gtk_source_mark_attributes_set_background (attrs, &color);
+
+	/* FIXME: same for GTK_STOCK_NO */
 	gtk_source_mark_attributes_set_stock_id (attrs, GTK_STOCK_NO);
 
 	g_signal_connect (attrs,
@@ -1338,14 +1176,20 @@ add_source_mark_pixbufs (GtkSourceView *view)
 }
 
 static GtkWidget *
-create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
+create_view_window (GtkSourceBuffer *buffer,
+		    GtkSourceView   *from)
 {
-	GtkWidget *window, *sw, *view, *vbox, *pos_label, *menu;
+	GtkWidget *window;
+	GtkWidget *sw;
+	GtkWidget *view;
+	GtkWidget *vbox;
+	GtkWidget *pos_label;
+	GtkWidget *menu;
 	PangoFontDescription *font_desc = NULL;
 	GtkAccelGroup *accel_group;
 	GtkActionGroup *action_group;
 	GtkUIManager *ui_manager;
-	GError *error;
+	GError *error = NULL;
 
 	g_return_val_if_fail (GTK_SOURCE_IS_BUFFER (buffer), NULL);
 	g_return_val_if_fail (from == NULL || GTK_SOURCE_IS_VIEW (from), NULL);
@@ -1358,15 +1202,36 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 
 	/* view */
 	view = gtk_source_view_new_with_buffer (buffer);
-	
-	if (style_scheme)
-		gtk_source_buffer_set_style_scheme (buffer, style_scheme);
 
-	g_signal_connect (buffer, "mark-set", G_CALLBACK (move_cursor_cb), view);
-	g_signal_connect (buffer, "changed", G_CALLBACK (update_cursor_position), view);
-	g_signal_connect (view, "line-mark-activated", G_CALLBACK (line_mark_activated), view);
-	g_signal_connect (window, "delete-event", (GCallback) window_deleted_cb, view);
-	g_signal_connect (buffer, "bracket-matched", G_CALLBACK (bracket_matched), 0);
+	if (style_scheme != NULL)
+	{
+		gtk_source_buffer_set_style_scheme (buffer, style_scheme);
+	}
+
+	g_signal_connect (buffer,
+			  "mark-set",
+			  G_CALLBACK (mark_set_cb),
+			  view);
+
+	g_signal_connect (buffer,
+			  "changed",
+			  G_CALLBACK (update_cursor_position_info),
+			  view);
+
+	g_signal_connect (view,
+			  "line-mark-activated",
+			  G_CALLBACK (line_mark_activated),
+			  view);
+
+	g_signal_connect (window,
+			  "delete-event",
+			  G_CALLBACK (window_deleted_cb),
+			  view);
+
+	g_signal_connect (buffer,
+			  "bracket-matched",
+			  G_CALLBACK (bracket_matched),
+			  NULL);
 
 	/* action group and UI manager */
 	action_group = gtk_action_group_new ("ViewActions");
@@ -1395,12 +1260,9 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 	accel_group = gtk_ui_manager_get_accel_group (ui_manager);
 	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
 
-	error = NULL;
 	if (!gtk_ui_manager_add_ui_from_string (ui_manager, view_ui_description, -1, &error))
 	{
-		g_message ("building view ui failed: %s", error->message);
-		g_error_free (error);
-		exit (1);
+		g_error ("building view ui failed: %s", error->message);
 	}
 
 	/* misc widgets */
@@ -1428,7 +1290,7 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 	}
 
 	/* change view attributes to match those of from */
-	if (from)
+	if (from != NULL)
 	{
 		gchar *tmp;
 		gint i;
@@ -1477,7 +1339,7 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 		g_free (tmp);
 	}
 
-	add_source_mark_pixbufs (GTK_SOURCE_VIEW (view));
+	add_source_mark_attributes (GTK_SOURCE_VIEW (view));
 
 	gtk_widget_show_all (vbox);
 
@@ -1492,7 +1354,7 @@ create_main_window (GtkSourceBuffer *buffer)
 	GtkUIManager *ui_manager;
 	GtkActionGroup *action_group;
 	GList *groups;
-	GError *error;
+	GError *error = NULL;
 
 	window = create_view_window (buffer, NULL);
 	ui_manager = g_object_get_data (G_OBJECT (window), "ui_manager");
@@ -1505,12 +1367,9 @@ create_main_window (GtkSourceBuffer *buffer)
 	g_object_unref (action_group);
 
 	/* merge buffer ui */
-	error = NULL;
 	if (!gtk_ui_manager_add_ui_from_string (ui_manager, buffer_ui_description, -1, &error))
 	{
-		g_message ("building buffer ui failed: %s", error->message);
-		g_error_free (error);
-		exit (1);
+		g_error ("building buffer ui failed: %s", error->message);
 	}
 
 	/* preselect menu checkitems */
@@ -1549,85 +1408,6 @@ create_main_window (GtkSourceBuffer *buffer)
 	return window;
 }
 
-
-/* XML memory management and verification functions ----------------------------- */
-
-#ifdef TEST_XML_MEM
-
-#define ALIGN 8
-
-/* my_free(malloc(n)) and free(my_malloc(n)) are invalid and
- * abort on glibc */
-
-static gpointer
-my_malloc (gsize n_bytes)
-{
-	char *mem = malloc (n_bytes + ALIGN);
-	return mem ? mem + ALIGN : NULL;
-}
-
-static gpointer
-my_realloc (gpointer mem,
-	    gsize    n_bytes)
-{
-	if (mem)
-	{
-		char *new_mem = realloc ((char*) mem - ALIGN, n_bytes + ALIGN);
-		return new_mem ? new_mem + ALIGN : NULL;
-	}
-	else
-	{
-		return my_malloc (n_bytes);
-	}
-}
-
-static char *
-my_strdup (const char *s)
-{
-	if (s)
-	{
-		char *new_s = my_malloc (strlen (s) + 1);
-		strcpy (new_s, s);
-		return new_s;
-	}
-	else
-	{
-		return NULL;
-	}
-}
-
-static void
-my_free (gpointer mem)
-{
-	if (mem)
-		free ((char*) mem - ALIGN);
-}
-
-static void
-init_mem_stuff (void)
-{
-	if (1)
-	{
-		if (xmlMemSetup (my_free, my_malloc, my_realloc, my_strdup) != 0)
-			g_warning ("xmlMemSetup() failed");
-	}
-	else
-	{
-		GMemVTable mem_table = {
-			my_malloc,
-			my_realloc,
-			my_free,
-			NULL, NULL, NULL
-		};
-
-		g_mem_set_vtable (&mem_table);
-		g_slice_set_config (G_SLICE_CONFIG_ALWAYS_MALLOC, TRUE);
-	}
-}
-
-#endif /* TEST_XML_MEM */
-
-
 /* Program entry point ------------------------------------------------------------ */
 
 int
@@ -1655,17 +1435,11 @@ main (int argc, char *argv[])
 	  { NULL }
 	};
 
-#ifdef TEST_XML_MEM
-	init_mem_stuff ();
-#endif
-
 	context = g_option_context_new ("- test GtkSourceView widget");
 	g_option_context_add_main_entries (context, entries, NULL);
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 	g_option_context_parse (context, &argc, &argv, NULL);
 
-// 	gdk_window_set_debug_updates (TRUE);
-
 	/* we do not use defaults so we don't need to install the library */
 	dirs = use_default_paths ? NULL : builtin_lang_dirs;
 	lm = gtk_source_language_manager_get_default ();
@@ -1677,7 +1451,9 @@ main (int argc, char *argv[])
 	gtk_source_style_scheme_manager_set_search_path (sm, dirs);
 
 	if (!use_default_paths)
+	{
 		gtk_source_style_scheme_manager_append_search_path (sm, TOP_SRCDIR "/tests/test-scheme.xml");
+	}
 
 	schemes = gtk_source_style_scheme_manager_get_scheme_ids (sm);
 	g_print ("Available style schemes:\n");
@@ -1690,7 +1466,9 @@ main (int argc, char *argv[])
 
 		authors = gtk_source_style_scheme_get_authors (style_scheme);
 		if (authors != NULL)
+		{
 			authors_str = g_strjoinv (", ", (gchar **)authors);
+		}
 
 		g_print (" - [%s] %s: %s\n",
 			 gtk_source_style_scheme_get_id (style_scheme),
@@ -1698,7 +1476,8 @@ main (int argc, char *argv[])
 			 gtk_source_style_scheme_get_description (style_scheme) ?
 				gtk_source_style_scheme_get_description (style_scheme) : "");
 
-		if (authors_str) {
+		if (authors_str != NULL)
+		{
 			g_print ("   by %s\n",  authors_str);
 			g_free (authors_str);
 		}
@@ -1708,9 +1487,13 @@ main (int argc, char *argv[])
 	g_print("\n");
 
 	if (style_scheme_id != NULL)
+	{
 		style_scheme = gtk_source_style_scheme_manager_get_scheme (sm, style_scheme_id);
+	}
 	else
+	{
 		style_scheme = NULL;
+	}
 
 	/* create buffer */
 	buffer = gtk_source_buffer_new (NULL);
@@ -1718,9 +1501,13 @@ main (int argc, char *argv[])
 	gtk_source_buffer_set_highlight_syntax (buffer, !no_syntax);
 
 	if (argc > 1)
+	{
 		open_file (buffer, argv [1]);
+	}
 	else
+	{
 		open_file (buffer, TOP_SRCDIR "/gtksourceview/gtksourcebuffer.c");
+	}
 
 	/* create first window */
 	window = create_main_window (buffer);
